@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21788 $
+ * $Revision: 24177 $
  * $Author: rjongbloed $
- * $Date: 2008-12-11 23:42:13 -0600 (Thu, 11 Dec 2008) $
+ * $Date: 2010-04-05 06:52:04 -0500 (Mon, 05 Apr 2010) $
  */
 
 #ifndef PTLIB_ASNER_H
@@ -65,13 +65,13 @@ class PASN_Object : public PObject
 
     PINDEX GetObjectLength() const;
     virtual PINDEX GetDataLength() const = 0;
-    virtual PBoolean IsPrimitive() const { return PTrue; }
+    virtual PBoolean IsPrimitive() const { return true; }
 
     virtual PBoolean Decode(PASN_Stream &) = 0;
     virtual void Encode(PASN_Stream &) const = 0;
 
     PBoolean IsExtendable() const { return extendable; }
-    void SetExtendable(PBoolean ext = PTrue) { extendable = ext; }
+    void SetExtendable(PBoolean ext = true) { extendable = ext; }
 
     enum TagClass {
       UniversalTagClass,
@@ -145,7 +145,7 @@ class PASN_Object : public PObject
     static void SetMaximumStringSize(PINDEX sz);
 
   protected:
-    PASN_Object(unsigned tag, TagClass tagClass, PBoolean extend = PFalse);
+    PASN_Object(unsigned tag, TagClass tagClass, PBoolean extend = false);
 
     /// PER extension capability
     PBoolean extendable;
@@ -207,8 +207,8 @@ class PASN_Boolean : public PASN_Object
 {
     PCLASSINFO(PASN_Boolean, PASN_Object);
   public:
-    PASN_Boolean(PBoolean val = PFalse);
-    PASN_Boolean(unsigned tag, TagClass tagClass, PBoolean val = PFalse);
+    PASN_Boolean(PBoolean val = false);
+    PASN_Boolean(unsigned tag, TagClass tagClass, PBoolean val = false);
 
     PASN_Boolean & operator=(PBoolean v) { value = v; return *this; }
     operator PBoolean() const { return value; }
@@ -279,7 +279,7 @@ class PASN_Enumeration : public PASN_Object
     PASN_Enumeration(unsigned tag,
                      TagClass tagClass,
                      unsigned nEnums = P_MAX_INDEX,
-                     PBoolean extendable = PFalse,
+                     PBoolean extendable = false,
                      unsigned val = 0);
     PASN_Enumeration(unsigned tag,
                      TagClass tagClass,
@@ -598,6 +598,8 @@ class PASN_BMPString : public PASN_ConstrainedObject
     void SetValue(const PString & v) { operator=(v.AsUCS2()); }
     void SetValue(const PWCharArray & v) { operator=(v); }
     void SetValue(const PASN_BMPString & v) { operator=(v.value); }
+    void SetValueRaw(const PWCharArray & v) { SetValueRaw(v, v.GetSize()); }
+    void SetValueRaw(const wchar_t * val, PINDEX len);
 
     void SetCharacterSet(ConstraintType ctype, const char * charSet);
     void SetCharacterSet(ConstraintType ctype, const PWCharArray & charSet);
@@ -642,8 +644,8 @@ class PASN_GeneralisedTime : public PASN_VisibleString
       : PASN_VisibleString(UniversalGeneralisedTime, UniversalTagClass) { }
     PASN_GeneralisedTime(const PTime & time)
       : PASN_VisibleString(UniversalGeneralisedTime, UniversalTagClass) { SetValue(time); }
-    PASN_GeneralisedTime(unsigned tag, TagClass tagClass)
-      : PASN_VisibleString(tag, tagClass) { }
+    PASN_GeneralisedTime(unsigned theTag, TagClass theTagClass)
+      : PASN_VisibleString(theTag, theTagClass) { }
 
     PASN_GeneralisedTime & operator=(const PTime & time);
     void SetValue(const PTime & time) { operator=(time); }
@@ -659,8 +661,8 @@ class PASN_UniversalTime : public PASN_VisibleString
       : PASN_VisibleString(UniversalUTCTime, UniversalTagClass) { }
     PASN_UniversalTime(const PTime & time)
       : PASN_VisibleString(UniversalUTCTime, UniversalTagClass) { SetValue(time); }
-    PASN_UniversalTime(unsigned tag, TagClass tagClass)
-      : PASN_VisibleString(tag, tagClass) { }
+    PASN_UniversalTime(unsigned theTag, TagClass theTagClass)
+      : PASN_VisibleString(theTag, theTagClass) { }
 
     PASN_UniversalTime & operator=(const PTime & time);
     void SetValue(const PTime & time) { operator=(time); }
@@ -762,7 +764,7 @@ class PASN_Choice : public PASN_Object
 
     PINDEX GetValueByName(PString name) const;
   protected:
-    PASN_Choice(unsigned nChoices = 0, PBoolean extend = PFalse);
+    PASN_Choice(unsigned nChoices = 0, PBoolean extend = false);
     PASN_Choice(unsigned tag, TagClass tagClass, unsigned nChoices, PBoolean extend);
     PASN_Choice(unsigned tag, TagClass tagClass, unsigned nChoices, PBoolean extend, const PASN_Names * nameSpec,unsigned namesCnt);
 
@@ -788,7 +790,7 @@ class PASN_Sequence : public PASN_Object
   public:
     PASN_Sequence(unsigned tag = UniversalSequence,
                   TagClass tagClass = UniversalTagClass,
-                  unsigned nOpts = 0, PBoolean extend = PFalse, unsigned nExtend = 0);
+                  unsigned nOpts = 0, PBoolean extend = false, unsigned nExtend = 0);
 
     PASN_Sequence(const PASN_Sequence & other);
     PASN_Sequence & operator=(const PASN_Sequence & other);
@@ -866,7 +868,7 @@ class PASN_Set : public PASN_Sequence
   public:
     PASN_Set(unsigned tag = UniversalSet,
              TagClass tagClass = UniversalTagClass,
-             unsigned nOpts = 0, PBoolean extend = PFalse, unsigned nExtend = 0);
+             unsigned nOpts = 0, PBoolean extend = false, unsigned nExtend = 0);
 
     virtual PObject * Clone() const;
     virtual PString GetTypeAsString() const;
