@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21788 $
+ * $Revision: 25012 $
  * $Author: rjongbloed $
- * $Date: 2008-12-12 05:42:13 +0000 (Fri, 12 Dec 2008) $
+ * $Date: 2011-01-06 01:01:23 -0600 (Thu, 06 Jan 2011) $
  */
 
 #ifndef PTLIB_FTP_H
@@ -40,6 +40,9 @@
 #include <ptlib/sockets.h>
 
 
+class PURL;
+
+
 /**
 File Transfer Protocol base class.
 */
@@ -74,6 +77,10 @@ class PFTP : public PInternetProtocol
       DetailedNames
     };
 
+    enum {
+      DefaultPort = 21
+    };
+
     /** Send the PORT command for a transfer.
      @return Boolean indicated PORT command was successful
     */
@@ -108,7 +115,7 @@ class PFTPClient : public PFTP
     /** Close the socket, and if connected as a client, QUITs from server.
 
        @return
-       PTrue if the channel was closed and the QUIT accepted by the server.
+       true if the channel was closed and the QUIT accepted by the server.
      */
     virtual PBoolean Close();
 
@@ -116,10 +123,17 @@ class PFTPClient : public PFTP
 
   /**@name New functions for class */
   //@{
+    /**Open host using TCP
+     */
+    bool OpenHost(
+      const PString & host,
+      WORD port = DefaultPort
+    );
+
     /** Log in to the remote host for FTP.
 
        @return
-       PTrue if the log in was successfull.
+       true if the log in was successfull.
      */
     PBoolean LogIn(
       const PString & username,   ///< User name for FTP log in.
@@ -136,7 +150,7 @@ class PFTPClient : public PFTP
     /** Set the transfer type.
 
        @return
-       PTrue if transfer type set.
+       true if transfer type set.
      */
     PBoolean SetType(
       RepresentationType type   ///< RepresentationTypeof file to transfer
@@ -145,7 +159,7 @@ class PFTPClient : public PFTP
     /** Change the current directory on the remote FTP host.
 
        @return
-       PTrue if the log in was successfull.
+       true if the log in was successfull.
      */
     PBoolean ChangeDirectory(
       const PString & dirPath     ///< New directory
@@ -183,7 +197,7 @@ class PFTPClient : public PFTP
     /** Create a directory on the remote FTP host.
 
        @return
-       PTrue if the directory was created successfully.
+       true if the directory was created successfully.
      */
     PBoolean CreateDirectory(
       const PString & path                ///< Name of the directory to create.
@@ -208,8 +222,8 @@ class PFTPClient : public PFTP
        Socket to read data from, or NULL if an error occurred.
      */
     PTCPSocket * GetFile(
-      const PString & filename,            ///< Name of file to get
-      DataChannelType channel = NormalPort ///< Data channel type.
+      const PString & filename,         ///< Name of file to get
+      DataChannelType channel = Passive ///< Data channel type.
     );
 
     /**Begin storing a file to the remote FTP server. The second parameter
@@ -221,8 +235,22 @@ class PFTPClient : public PFTP
        Socket to write data to, or NULL if an error occurred.
      */
     PTCPSocket * PutFile(
-      const PString & filename,   ///< Name of file to get
-      DataChannelType channel = NormalPort ///< Data channel type.
+      const PString & filename,         ///< Name of file to get
+      DataChannelType channel = Passive ///< Data channel type.
+    );
+
+    /**Begin retreiving a file from the remote FTP server. The second
+       parameter indicates that the transfer is on a normal or passive data
+       channel. In short, a normal transfer the server connects to the
+       client and in passive mode the client connects to the server.
+
+       @return
+       Socket to read data from, or NULL if an error occurred.
+     */
+    PTCPSocket * GetURL(
+      const PURL & url,                 ///< URL of file to get
+      RepresentationType type,          ///< Type of transfer (text/binary)
+      DataChannelType channel = Passive ///< Data channel type.
     );
 
   //@}
@@ -287,8 +315,8 @@ class PFTPServer : public PFTP
        is used when the socket is acting as a server.
 
        @return
-       PTrue if more processing may be done, PFalse if the QUIT command was
-       received or the #OnUnknown()# function returns PFalse.
+       true if more processing may be done, false if the QUIT command was
+       received or the <code>OnUnknown()</code> function returns false.
      */
     PBoolean ProcessCommand();
 
@@ -296,8 +324,8 @@ class PFTPServer : public PFTP
        socket is acting as a server.
 
        @return
-       PTrue if more processing may be done, PFalse if the QUIT command was
-       received or the #OnUnknown()# function returns PFalse.
+       true if more processing may be done, false if the QUIT command was
+       received or the <code>OnUnknown()</code> function returns false.
      */
     virtual PBoolean DispatchCommand(
       PINDEX code,          ///< Parsed command code.
@@ -309,19 +337,19 @@ class PFTPServer : public PFTP
        it may be processed.
 
        @return
-       PTrue if the command required the user to be logged in.
+       true if the command required the user to be logged in.
      */
     virtual PBoolean CheckLoginRequired(
       PINDEX cmd    ///< Command to check if log in required.
     );
 
     /** Validate the user name and password for access. After three invalid
-       attempts, the socket will close and PFalse is returned.
+       attempts, the socket will close and false is returned.
 
-       Default implementation returns PTrue for all strings.
+       Default implementation returns true for all strings.
 
        @return
-       PTrue if user can access, otherwise PFalse
+       true if user can access, otherwise false
      */
     virtual PBoolean AuthoriseUser(
       const PString & user,     ///< User name to authorise.
@@ -332,8 +360,8 @@ class PFTPServer : public PFTP
     /** Handle an unknown command.
 
        @return
-       PTrue if more processing may be done, PFalse if the
-       #ProcessCommand()# function is to return PFalse.
+       true if more processing may be done, false if the
+       <code>ProcessCommand()</code> function is to return false.
      */
     virtual PBoolean OnUnknown(
       const PCaselessString & command  ///< Complete command line received.
@@ -342,8 +370,8 @@ class PFTPServer : public PFTP
     /** Handle an error in command.
 
        @return
-       PTrue if more processing may be done, PFalse if the
-       #ProcessCommand()# function is to return PFalse.
+       true if more processing may be done, false if the
+       <code>ProcessCommand()</code> function is to return false.
      */
     virtual void OnError(
       PINDEX errorCode, ///< Error code to use
@@ -448,6 +476,9 @@ class PFTPServer : public PFTP
 };
 
 
+PFACTORY_LOAD(PURL_FtpLoader);
+
+
 #endif // PTLIB_FTP_H
 
 
