@@ -23,13 +23,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
+ * $Revision: 24177 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 10:40:38 +0000 (Wed, 04 Jun 2008) $
+ * $Date: 2010-04-05 06:52:04 -0500 (Mon, 05 Apr 2010) $
  */
 
-#ifndef _PHTTPFORM_H
-#define _PHTTPFORM_H
+#ifndef PTLIB_HTTPFORM_H
+#define PTLIB_HTTPFORM_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
@@ -44,7 +44,7 @@
 ///////////////////////////////////////////////////////////////////////////////
 // PHTTPField
 
-/** This class is the abstract base class for fields in a #PHTTPForm#
+/** This class is the abstract base class for fields in a <code>PHTTPForm</code>
    resource type.
  */
 class PHTTPField : public PObject
@@ -163,7 +163,7 @@ class PHTTPField : public PObject
        @return
        String for field value.
      */
-    virtual PString GetValue(PBoolean dflt = PFalse) const = 0;
+    virtual PString GetValue(PBoolean dflt = false) const = 0;
 
     /** Set the value of the field.
      */
@@ -185,7 +185,7 @@ class PHTTPField : public PObject
       PConfig & cfg   // Configuration for value transfer.
     ) const;
 
-    /** Validate the new field value before #SetValue()# is called.
+    /** Validate the new field value before <code>SetValue()</code> is called.
 
        @return
        PBoolean if the new field value is OK.
@@ -209,7 +209,7 @@ class PHTTPField : public PObject
       const PStringToString & data   // New value for the field.
     );
 
-    /** Validate the new field value in a list before #SetValue()# is called.
+    /** Validate the new field value in a list before <code>SetValue()</code> is called.
 
        @return
        PBoolean if the all the new field values are OK.
@@ -221,7 +221,7 @@ class PHTTPField : public PObject
 
 
     PBoolean NotYetInHTML() const { return notInHTML; }
-    void SetInHTML() { notInHTML = PFalse; }
+    void SetInHTML() { notInHTML = false; }
 
   protected:
     PCaselessString baseName;
@@ -241,7 +241,8 @@ class PHTTPCompositeField : public PHTTPField
     PHTTPCompositeField(
       const char * name,          // Name (identifier) for the field.
       const char * title = NULL,  // Title text for field (defaults to name).
-      const char * help = NULL    // Help text for the field.
+      const char * help = NULL,   // Help text for the field.
+      bool includeHeaders = false // Make a sub-table and put headers on HTML fields.
     );
 
     virtual void SetName(
@@ -268,7 +269,7 @@ class PHTTPCompositeField : public PHTTPField
       PHTML & html    // HTML to receive the field info.
     ) const;
 
-    virtual PString GetValue(PBoolean dflt = PFalse) const;
+    virtual PString GetValue(PBoolean dflt = false) const;
 
     virtual void SetValue(
       const PString & newValue   // New value for the field.
@@ -308,6 +309,7 @@ class PHTTPCompositeField : public PHTTPField
 
   protected:
     PHTTPFields fields;
+    bool        m_includeHeaders;
 };
 
 
@@ -414,7 +416,7 @@ class PHTTPStringField : public PHTTPField
       PHTML & html    ///< HTML to receive the field info.
     ) const;
 
-    virtual PString GetValue(PBoolean dflt = PFalse) const;
+    virtual PString GetValue(PBoolean dflt = false) const;
 
     virtual void SetValue(
       const PString & newVal
@@ -452,7 +454,7 @@ class PHTTPPasswordField : public PHTTPStringField
       PHTML & html    ///< HTML to receive the field info.
     ) const;
 
-    virtual PString GetValue(PBoolean dflt = PFalse) const;
+    virtual PString GetValue(PBoolean dflt = false) const;
 
     virtual void SetValue(
       const PString & newVal
@@ -462,6 +464,32 @@ class PHTTPPasswordField : public PHTTPStringField
 };
 
 
+class PHTTPDateField : public PHTTPStringField
+{
+  PCLASSINFO(PHTTPDateField, PHTTPStringField)
+  public:
+    PHTTPDateField(
+      const char * name,
+      const PTime & initVal = PTime(0),
+      PTime::TimeFormat fmt = PTime::ShortDate
+    );
+
+    virtual PHTTPField * NewField() const;
+
+    virtual void SetValue(
+      const PString & newValue
+    );
+
+    virtual PBoolean Validated(
+      const PString & newValue,
+      PStringStream & msg
+    ) const;
+
+  protected:
+    PTime::TimeFormat m_format;
+};
+
+
 class PHTTPIntegerField : public PHTTPField
 {
   PCLASSINFO(PHTTPIntegerField, PHTTPField)
@@ -488,7 +516,7 @@ class PHTTPIntegerField : public PHTTPField
       PHTML & html    ///< HTML to receive the field info.
     ) const;
 
-    virtual PString GetValue(PBoolean dflt = PFalse) const;
+    virtual PString GetValue(PBoolean dflt = false) const;
 
     virtual void SetValue(
       const PString & newVal
@@ -520,13 +548,13 @@ class PHTTPBooleanField : public PHTTPField
   public:
     PHTTPBooleanField(
       const char * name,
-      PBoolean initVal = PFalse,
+      PBoolean initVal = false,
       const char * help = NULL
     );
     PHTTPBooleanField(
       const char * name,
       const char * title,
-      PBoolean initVal = PFalse,
+      PBoolean initVal = false,
       const char * help = NULL
     );
 
@@ -540,7 +568,7 @@ class PHTTPBooleanField : public PHTTPField
       const PString & input
     ) const;
 
-    virtual PString GetValue(PBoolean dflt = PFalse) const;
+    virtual PString GetValue(PBoolean dflt = false) const;
 
     virtual void SetValue(
       const PString & newVal
@@ -634,7 +662,7 @@ class PHTTPRadioField : public PHTTPField
       const PString & input
     ) const;
 
-    virtual PString GetValue(PBoolean dflt = PFalse) const;
+    virtual PString GetValue(PBoolean dflt = false) const;
 
     virtual void SetValue(
       const PString & newVal
@@ -688,7 +716,7 @@ class PHTTPSelectField : public PHTTPField
       PHTML & html    ///< HTML to receive the field info.
     ) const;
 
-    virtual PString GetValue(PBoolean dflt = PFalse) const;
+    virtual PString GetValue(PBoolean dflt = false) const;
 
     virtual void SetValue(
       const PString & newVal
@@ -896,7 +924,7 @@ class PHTTPConfigSectionList : public PHTTPString
 
 #endif // P_HTTPFORMS
 
-#endif // _HTTPFORM_H
+#endif // PTLIB_HTTPFORM_H
 
 
 // End Of File ///////////////////////////////////////////////////////////////
