@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21788 $
+ * $Revision: 27535 $
  * $Author: rjongbloed $
- * $Date: 2008-12-12 05:42:13 +0000 (Fri, 12 Dec 2008) $
+ * $Date: 2012-04-26 02:48:22 -0500 (Thu, 26 Apr 2012) $
  */
 
 #ifndef PTLIB_PLDAP_H
@@ -111,6 +111,10 @@ class PLDAPSession : public PObject
 #endif
     };
 
+    /**Start encrypted connection
+      */
+    PBoolean StartTLS();
+
     /**Bind to the remote LDAP server.
       */
     PBoolean Bind(
@@ -462,7 +466,7 @@ class PLDAPStructBase : public PObject {
     PLDAPStructBase & operator=(const PStringArray & array);
     PLDAPStructBase & operator=(const PStringToString & dict);
   private:
-    PLDAPStructBase(const PLDAPStructBase &) { }
+    PLDAPStructBase(const PLDAPStructBase & obj) : PObject(obj) { }
 
   public:
     void PrintOn(ostream & strm) const;
@@ -556,8 +560,7 @@ template <class className> class LDAPPluginServiceDescriptor : public PDevicePlu
 
 #define LDAP_Schema(name)    \
   static LDAPPluginServiceDescriptor<name##_schema> name##_schema_descriptor; \
-  PCREATE_PLUGIN(name##_schema, PLDAPSchema, &name##_schema_descriptor); \
-  PWLIB_STATIC_LOAD_PLUGIN(name##_schema, PLDAPSchema); \
+  PCREATE_PLUGIN(name##_schema, PLDAPSchema, &name##_schema_descriptor)
 
 ////////////////////////////////////////////////////////////////////////////////////////////
 
