@@ -26,17 +26,16 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
- * $Author: rjongbloed $
- * $Date: 2008-06-04 05:40:38 -0500 (Wed, 04 Jun 2008) $
+ * $Revision: 26863 $
+ * $Author: ededu $
+ * $Date: 2012-01-17 09:04:17 -0600 (Tue, 17 Jan 2012) $
  */
 
+#ifndef EINPROGRESS
 #define EINPROGRESS             (WSAEINPROGRESS|PWIN32ErrorFlag)
 #define ENOTSOCK                (WSAENOTSOCK|PWIN32ErrorFlag)
 #define EMSGSIZE                (WSAEMSGSIZE|PWIN32ErrorFlag)
-#define ESOCKTNOSUPPORT         (WSAESOCKTNOSUPPORT|PWIN32ErrorFlag)
 #define EOPNOTSUPP              (WSAEOPNOTSUPP|PWIN32ErrorFlag)
-#define EPFNOSUPPORT            (WSAEPFNOSUPPORT|PWIN32ErrorFlag)
 #define EAFNOSUPPORT            (WSAEAFNOSUPPORT|PWIN32ErrorFlag)
 #define EADDRINUSE              (WSAEADDRINUSE|PWIN32ErrorFlag)
 #define EADDRNOTAVAIL           (WSAEADDRNOTAVAIL|PWIN32ErrorFlag)
@@ -48,12 +47,11 @@
 #define ENOBUFS                 (WSAENOBUFS|PWIN32ErrorFlag)
 #define EISCONN                 (WSAEISCONN|PWIN32ErrorFlag)
 #define ENOTCONN                (WSAENOTCONN|PWIN32ErrorFlag)
-#define ESHUTDOWN               (WSAESHUTDOWN|PWIN32ErrorFlag)
-#define ETOOMANYREFS            (WSAETOOMANYREFS|PWIN32ErrorFlag)
 #define ETIMEDOUT               (WSAETIMEDOUT|PWIN32ErrorFlag)
 #define ECONNREFUSED            (WSAECONNREFUSED|PWIN32ErrorFlag)
-#define EHOSTDOWN               (WSAEHOSTDOWN|PWIN32ErrorFlag)
 #define EHOSTUNREACH            (WSAEHOSTUNREACH|PWIN32ErrorFlag)
+#define EWOULDBLOCK             (WSAEWOULDBLOCK|PWIN32ErrorFlag)
+#endif
 
 
 #define NETDB_SUCCESS 0
