@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25079 $
+ * $Revision: 28275 $
  * $Author: rjongbloed $
- * $Date: 2011-01-20 12:23:32 -0600 (Thu, 20 Jan 2011) $
+ * $Date: 2012-08-29 21:42:35 -0500 (Wed, 29 Aug 2012) $
  */
 
 
@@ -173,6 +173,8 @@ class PSoundChannelWin32: public PSoundChannel
     PBoolean Abort();
     PBoolean SetVolume(unsigned newVal);
     PBoolean GetVolume(unsigned &devVol);
+    PBoolean SetMute(bool mute);
+    PBoolean GetMute(bool & mute);
 
   public:
     // Overrides from class PChannel
@@ -195,6 +197,7 @@ class PSoundChannelWin32: public PSoundChannel
     HWAVEOUT     hWaveOut;
     HMIXER       hMixer;
     MIXERCONTROL volumeControl;
+    MIXERCONTROL muteControl;
     HANDLE       hEventDone;
     PWaveFormat  waveFormat;
     bool         opened;
