@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24284 $
+ * $Revision: 27977 $
  * $Author: rjongbloed $
- * $Date: 2010-04-26 00:55:15 -0500 (Mon, 26 Apr 2010) $
+ * $Date: 2012-07-09 03:53:24 -0500 (Mon, 09 Jul 2012) $
  */
 
 
@@ -41,12 +41,8 @@
 
   protected:
     HANDLE threadHandle;
-    UINT   threadId;
 
   private:
-    bool   m_isProcess;
-    PINDEX originalStackSize;
-
     void CleanUp();
     static UINT __stdcall MainFunction(void * thread);
 
