@@ -27,9 +27,9 @@
  *
  * Contributor(s): /
  *
- * $Revision: 25459 $
- * $Author: csoutheren $
- * $Date: 2011-03-30 18:10:26 -0500 (Wed, 30 Mar 2011) $
+ * $Revision: 28821 $
+ * $Author: ededu $
+ * $Date: 2013-01-09 04:43:50 -0600 (Wed, 09 Jan 2013) $
  */
 
 #pragma implementation "sound_alsa.h"
@@ -132,24 +132,15 @@ void PSoundChannelALSA::UpdateDictionary(Directions dir)
           if (rawName != NULL) {
             int disambiguator = 1;
             PString uniqueName = rawName;
+            uniqueName = uniqueName + " [" + snd_pcm_info_get_name (pcminfo) + "]";
+            PString save = uniqueName;
             while (devices.Contains(uniqueName)) {
-              uniqueName = rawName;
+              uniqueName = save;
               uniqueName.sprintf(" (%d)", disambiguator++);
             }
 
             devices.SetAt(uniqueName, cardNum);
-
-#ifdef snd_card_get_name_allocates_on_the_heap
-            /*snd_card_get_name is unclear on if the pointer returned is
-              static or on the heap. A web search shows that about half
-              the implemetations out there do the free() and half don't!
-
-              As it is causing at least one user a problem with "Block XXXX
-              not in heap!" errors being output to stderr, we take it out and
-              suffer the memory leak, if there is one ....
-             */
             free(rawName);
-#endif
           }
         }
       }
@@ -403,21 +394,25 @@ PBoolean PSoundChannelALSA::Write(const void *buf, PINDEX len)
       lastWriteCount += r * frameBytes;
     }
     else {
+      PTRACE(5, "ALSA\tBuffer underrun detected. Recovering... ");
       if (r == -EPIPE) {    /* under-run */
         r = snd_pcm_prepare(os_handle);
-        if (r < 0) {
-          PTRACE(1, "ALSA\tCould not prepare device: " << snd_strerror(r));
-        }
+        PTRACE_IF(1, r < 0, "ALSA\tCould not prepare device: " << snd_strerror(r));
       }
       else if (r == -ESTRPIPE) {
+        PTRACE(5, "ALSA\tOutput suspended. Resuming... ");
         while ((r = snd_pcm_resume(os_handle)) == -EAGAIN)
           sleep(1);       /* wait until the suspend flag is released */
 
-        if (r < 0)
-          snd_pcm_prepare(os_handle);
+        if (r < 0) {
+          r = snd_pcm_prepare(os_handle);
+          PTRACE_IF(1, r < 0, "ALSA\tCould not prepare device: " << snd_strerror(r));
+        }
+      }
+      else {
+        PTRACE(1, "ALSA\tCould not write " << max_try << " " << len << " " << snd_strerror(r));
       }
 
-      PTRACE(1, "ALSA\tCould not write " << max_try << " " << len << " " << snd_strerror(r));
       max_try++;
       if (max_try > 5)
         return false;
