@@ -24,9 +24,9 @@
  * Contributor(s): Derek Smithies (derek@indranet.co.nz)
  *                 Mark Cooke (mpc@star.sr.bham.ac.uk)
  *
- * $Revision: 20385 $
- * $Author: rjongbloed $
- * $Date: 2008-06-04 05:40:38 -0500 (Wed, 04 Jun 2008) $
+ * $Revision: 25888 $
+ * $Author: ededu $
+ * $Date: 2011-05-24 05:07:25 -0500 (Tue, 24 May 2011) $
  */
 
 #pragma implementation "vidinput_v4l.h"
@@ -322,7 +322,15 @@ void  V4LNames::ReadDeviceDirectory(PDirectory devdir, POrdinalToString & vid)
       if (devdir.GetInfo(info) && info.type == PFileInfo::CharDevice) {
         struct stat s;
         if (lstat(devname, &s) == 0) {
- 
+#if defined(P_FREEBSD)
+          // device numbers are irrelevant here, so we match on names instead
+          if (filename.GetLength() <= 5 || filename.Left(5) != "video")
+            continue;
+          int num = atoi(filename.Mid(6));
+          if (num < 0 || num > 63)
+            continue;
+          vid.SetAt(num, devname);
+#else
           static const int deviceNumbers[] = { 81 };
           for (PINDEX i = 0; i < PARRAYSIZE(deviceNumbers); i++) {
             if (MAJOR(s.st_rdev) == deviceNumbers[i]) {
@@ -332,6 +340,7 @@ void  V4LNames::ReadDeviceDirectory(PDirectory devdir, POrdinalToString & vid)
               }
             }
           }
+#endif
         }
       }
     }
@@ -625,7 +634,10 @@ PBoolean PVideoInputDevice_V4L::Open(const PString & devName, PBoolean startImme
     videoAudio.flags &= ~VIDEO_AUDIO_MUTE;
     videoAudio.mode = VIDEO_SOUND_MONO;
     ::ioctl(videoFd, VIDIOCSAUDIO, &videoAudio);
-    } 
+    }
+
+  SetVideoFormat(videoFormat);
+  SetColourFormat(colourFormat);
 
   return PTrue;
 }
