@@ -30,9 +30,9 @@
  * Contributor(s): Guilhem Tardy (gtardy@salyens.com)
  *  Nicola Orru' <nigu@itadinanta.it>
  *
- * $Revision: 27739 $
- * $Author: rjongbloed $
- * $Date: 2012-05-30 18:49:09 -0500 (Wed, 30 May 2012) $
+ * $Revision: 29220 $
+ * $Author: ededu $
+ * $Date: 2013-03-07 12:22:08 -0600 (Thu, 07 Mar 2013) $
  */
 
 #pragma implementation "vidinput_v4l2.h"
@@ -219,9 +219,10 @@ PBoolean PVideoInputDevice_V4L2::Open(const PString & devName, PBoolean /* start
     canSetFrameRate = videoStreamParm.parm.capture.capability & V4L2_CAP_TIMEPERFRAME;
     if (canSetFrameRate) {
       if (videoStreamParm.parm.capture.timeperframe.numerator == 0) {
-        PTRACE(1,"PVidInDev\tnumerator is zero and denominator is " << videoStreamParm.parm.capture.timeperframe.denominator << ", driver bug??");
-      }
-      PVideoDevice::SetFrameRate (videoStreamParm.parm.capture.timeperframe.denominator / videoStreamParm.parm.capture.timeperframe.numerator);
+        PTRACE(1,"PVidInDev\tDriver/webcam bug: numerator is zero and denominator is " << videoStreamParm.parm.capture.timeperframe.denominator << ", I assume it cannot set frame rate");
+        canSetFrameRate = PFalse;
+      } else
+        PVideoDevice::SetFrameRate (videoStreamParm.parm.capture.timeperframe.denominator / videoStreamParm.parm.capture.timeperframe.numerator);
     }
   }
 
