@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26555 $
+ * $Revision: 29065 $
  * $Author: rjongbloed $
- * $Date: 2011-10-06 21:49:36 -0500 (Thu, 06 Oct 2011) $
+ * $Date: 2013-02-11 18:26:26 -0600 (Mon, 11 Feb 2013) $
  */
 
 #include <ptlib.h>
@@ -157,19 +157,19 @@ int PHTTPClient::ExecuteCommand(const PString & cmdName,
 }
 
 
-PBoolean PHTTPClient::WriteCommand(Commands cmd,
-                               const PString & url,
-                               PMIMEInfo & outMIME,
-                               const PString & dataBody)
+bool PHTTPClient::WriteCommand(Commands cmd,
+                        const PString & url,
+                            PMIMEInfo & outMIME,
+                        const PString & dataBody)
 {
   return WriteCommand(commandNames[cmd], url, outMIME, dataBody);
 }
 
 
-PBoolean PHTTPClient::WriteCommand(const PString & cmdName,
-                                   const PString & url,
-                                       PMIMEInfo & outMIME,
-                                   const PString & dataBody)
+bool PHTTPClient::WriteCommand(const PString & cmdName,
+                               const PString & url,
+                                   PMIMEInfo & outMIME,
+                               const PString & dataBody)
 {
   ostream & this_stream = *this;
   PINDEX len = dataBody.GetLength();
@@ -214,7 +214,7 @@ PBoolean PHTTPClient::WriteCommand(const PString & cmdName,
 }
 
 
-PBoolean PHTTPClient::ReadResponse(PMIMEInfo & replyMIME)
+bool PHTTPClient::ReadResponse(PMIMEInfo & replyMIME)
 {
   PString http = ReadString(7);
   if (!http) {
@@ -279,7 +279,13 @@ PBoolean PHTTPClient::ReadResponse(PMIMEInfo & replyMIME)
 }
 
 
-PBoolean PHTTPClient::ReadContentBody(PMIMEInfo & replyMIME, PString & body)
+bool PHTTPClient::ReadContentBody(PMIMEInfo & replyMIME)
+{
+  return InternalReadContentBody(replyMIME, NULL);
+}
+
+
+bool PHTTPClient::ReadContentBody(PMIMEInfo & replyMIME, PString & body)
 {
   PCharArray rawBody;
   if (!InternalReadContentBody(replyMIME, &rawBody))
@@ -290,13 +296,13 @@ PBoolean PHTTPClient::ReadContentBody(PMIMEInfo & replyMIME, PString & body)
 }
 
 
-PBoolean PHTTPClient::ReadContentBody(PMIMEInfo & replyMIME, PBYTEArray & body)
+bool PHTTPClient::ReadContentBody(PMIMEInfo & replyMIME, PBYTEArray & body)
 {
   return InternalReadContentBody(replyMIME, &body);
 }
 
 
-PBoolean PHTTPClient::InternalReadContentBody(PMIMEInfo & replyMIME, PAbstractArray * body)
+bool PHTTPClient::InternalReadContentBody(PMIMEInfo & replyMIME, PAbstractArray * body)
 {
   PCaselessString encoding = replyMIME(TransferEncodingTag());
 
@@ -378,9 +384,9 @@ PBoolean PHTTPClient::InternalReadContentBody(PMIMEInfo & replyMIME, PAbstractAr
 }
 
 
-PBoolean PHTTPClient::GetTextDocument(const PURL & url,
-                                      PString & document,
-                                      const PString & requiredContentType)
+bool PHTTPClient::GetTextDocument(const PURL & url,
+                                     PString & document,
+                               const PString & requiredContentType)
 {
   PMIMEInfo outMIME, replyMIME;
   if (!GetDocument(url, outMIME, replyMIME))
@@ -405,42 +411,60 @@ PBoolean PHTTPClient::GetTextDocument(const PURL & url,
 }
 
 
-PBoolean PHTTPClient::GetDocument(const PURL & url,
-                              PMIMEInfo & outMIME,
-                              PMIMEInfo & replyMIME)
+bool PHTTPClient::GetDocument(const PURL & url, PMIMEInfo & outMIME, PMIMEInfo & replyMIME)
 {
   return IsOK(ExecuteCommand(GET, url, outMIME, PString::Empty(), replyMIME));
 }
 
 
-PBoolean PHTTPClient::GetHeader(const PURL & url,
-                            PMIMEInfo & outMIME,
-                            PMIMEInfo & replyMIME)
+bool PHTTPClient::GetDocument(const PURL & url, PMIMEInfo & replyMIME)
+{
+  PMIMEInfo outMIME;
+  return IsOK(ExecuteCommand(GET, url, outMIME, PString::Empty(), replyMIME));
+}
+
+
+bool PHTTPClient::GetHeader(const PURL & url, PMIMEInfo & outMIME, PMIMEInfo & replyMIME)
 {
   return IsOK(ExecuteCommand(HEAD, url, outMIME, PString::Empty(), replyMIME));
 }
 
 
-PBoolean PHTTPClient::PostData(const PURL & url,
-                           PMIMEInfo & outMIME,
-                           const PString & data,
-                           PMIMEInfo & replyMIME)
+bool PHTTPClient::PostData(const PURL & url, const PStringToString & data)
 {
-  PString dataBody = data;
-  if (!outMIME.Contains(ContentTypeTag())) {
+  PStringStream entityBody;
+  PURL::OutputVars(entityBody, data, '\0', '&', '=', PURL::QueryTranslation);
+  entityBody << "\r\n"; // Add CRLF for compatibility with some CGI servers.
+
+  PMIMEInfo outMIME;
+  return PostData(url, outMIME, entityBody);
+}
+
+
+bool PHTTPClient::PostData(const PURL & url, PMIMEInfo & outMIME, const PString & data)
+{
+  PMIMEInfo replyMIME;
+  return PostData(url, outMIME, data, replyMIME) && ReadContentBody(replyMIME);
+}
+
+
+bool PHTTPClient::PostData(const PURL & url,
+                            PMIMEInfo & outMIME,
+                        const PString & data,
+                            PMIMEInfo & replyMIME)
+{
+  if (!outMIME.Contains(ContentTypeTag()))
     outMIME.SetAt(ContentTypeTag(), "application/x-www-form-urlencoded");
-    dataBody += "\r\n"; // Add CRLF for compatibility with some CGI servers.
-  }
 
   return IsOK(ExecuteCommand(POST, url, outMIME, data, replyMIME));
 }
 
 
-PBoolean PHTTPClient::PostData(const PURL & url,
-                           PMIMEInfo & outMIME,
-                           const PString & data,
-                           PMIMEInfo & replyMIME,
-                           PString & body)
+bool PHTTPClient::PostData(const PURL & url,
+                            PMIMEInfo & outMIME,
+                        const PString & data,
+                            PMIMEInfo & replyMIME,
+                              PString & body)
 {
   return PostData(url, outMIME, data, replyMIME) && ReadContentBody(replyMIME, body);
 }
@@ -471,7 +495,7 @@ bool PHTTPClient::DeleteDocument(const PURL & url)
 }
 
 
-PBoolean PHTTPClient::AssureConnect(const PURL & url, PMIMEInfo & outMIME)
+bool PHTTPClient::AssureConnect(const PURL & url, PMIMEInfo & outMIME)
 {
   PString host = url.GetHostName();
 
@@ -832,7 +856,7 @@ PBoolean PHTTPClientDigestAuthentication::Authorise(AuthObject & authObject) con
          << "response=\"" << AsHex(response) << "\", "
          << "cnonce=\"" << cnonce << "\", "
          << "nc=" << nc << ", "
-         << "qop=\"" << qop << '"';
+         << "qop=" << qop;
   }
   else {
     digestor.Process(AsHex(a2));
