@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25392 $
+ * $Revision: 29189 $
  * $Author: rjongbloed $
- * $Date: 2011-03-22 23:29:41 -0500 (Tue, 22 Mar 2011) $
+ * $Date: 2013-03-03 18:06:07 -0600 (Sun, 03 Mar 2013) $
  */
 
 #include <ptlib.h>
@@ -165,7 +165,7 @@ PBoolean PHTTPSpace::DelResource(const PURL & url)
       Node * par = node->parent;
       par->children.Remove(node);
       node = par;
-    } while (node != NULL && node->children.IsEmpty());
+    } while (node->parent != NULL && node->children.IsEmpty());
   }
 
   return PTrue;
