@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
- * $Author: rjongbloed $
- * $Date: 2008-06-04 10:40:38 +0000 (Wed, 04 Jun 2008) $
+ * $Revision: 24854 $
+ * $Author: csoutheren $
+ * $Date: 2010-11-01 09:35:07 -0500 (Mon, 01 Nov 2010) $
  */
 
 #ifdef __GNUC__
@@ -60,9 +60,6 @@ class PServiceMacros_list : public PServiceMacros_base
 
 static const PTime ImmediateExpiryTime(0, 0, 0, 1, 1, 1980);
 
-
-///////////////////////////////////////////////////////////////////////////////
-
 PHTTPServiceProcess::PHTTPServiceProcess(const Info & inf)
   : PServiceProcess(inf.manufacturerName, inf.productName,
                     inf.majorVersion, inf.minorVersion, inf.buildStatus, inf.buildNumber),
@@ -90,7 +87,7 @@ PHTTPServiceProcess::PHTTPServiceProcess(const Info & inf)
 #endif
     httpNameSpace.AddResource(new PServiceHTTPFile(inf.gifFilename, exeDir+inf.gifFilename));
     if (gifHTML.IsEmpty()) {
-      gifHTML = psprintf("<img src=\"/%s\" alt=\"%s!\"", inf.gifFilename, inf.productName);
+      gifHTML = psprintf("<img border=0 src=\"%s\" alt=\"%s!\"", inf.gifFilename, inf.productName);
       if (inf.gifWidth != 0 && inf.gifHeight != 0)
         gifHTML += psprintf(" width=%i height=%i", inf.gifWidth, inf.gifHeight);
       gifHTML += " align=absmiddle>";
@@ -242,21 +239,19 @@ PString PHTTPServiceProcess::GetCopyrightText()
 
 PString PHTTPServiceProcess::GetPageGraphic()
 {
-  PFile header;
-  if (header.Open("header.html", PFile::ReadOnly))
-    return header.ReadString(header.GetLength());
-
   PHTML html(PHTML::InBody);
   html << PHTML::TableStart()
        << PHTML::TableRow()
-       << PHTML::TableData();
+       << PHTML::TableData()
 
+       << PHTML::HotLink("/");
   if (gifHTML.IsEmpty())
     html << PHTML::Heading(1) << productNameHTML << "&nbsp;" << PHTML::Heading(1);
   else
     html << gifHTML;
+  html << PHTML::HotLink()
 
-  html << PHTML::TableData()
+       << PHTML::TableData()
        << GetOSClass() << ' ' << GetOSName()
        << " Version " << GetVersion(PTrue) << PHTML::BreakLine()
        << ' ' << GetCompilationDate().AsString("d MMMM yyyy")
@@ -1571,7 +1566,7 @@ static void SplitCmdAndArgs(const PString & text, PINDEX pos, PCaselessString &
 }
 
 
-PBoolean PServiceHTML::ProcessMacros(PHTTPRequest & request,
+bool PServiceHTML::ProcessMacros(PHTTPRequest & request,
                                  PString & text,
                                  const PString & defaultFile,
                                  unsigned options)
@@ -1641,15 +1636,16 @@ PBoolean PServiceHTML::ProcessMacros(PHTTPRequest & request,
       PCaselessString cmd;
       PString args;
       SplitCmdAndArgs(text, pos, cmd, args);
-      PINDEX idx = ServiceMacros.GetValuesIndex(PServiceMacro(cmd, PTrue));
-      if (idx != P_MAX_INDEX) {
-        PRegularExpression EndBlockRegEx("<?!--#(equival|" + process.GetMacroKeyword() + ")"
-                                         "end[ \t\r\n]+" + cmd + "(-?[^-])*-->?",
-                                         PRegularExpression::Extended|PRegularExpression::IgnoreCase);
-        PINDEX endpos, endlen;
-        if (text.FindRegEx(EndBlockRegEx, endpos, endlen, pos+len)) {
-          PINDEX startpos = pos+len;
-          len = endpos-pos + endlen;
+
+      PRegularExpression EndBlockRegEx("<?!--#(equival|" + process.GetMacroKeyword() + ")"
+                                       "end[ \t\r\n]+" + cmd + "(-?[^-])*-->?",
+                                       PRegularExpression::Extended|PRegularExpression::IgnoreCase);
+      PINDEX endpos, endlen;
+      if (text.FindRegEx(EndBlockRegEx, endpos, endlen, pos+len)) {
+        PINDEX startpos = pos+len;
+        len = endpos-pos + endlen;
+        PINDEX idx = ServiceMacros.GetValuesIndex(PServiceMacro(cmd, PTrue));
+        if (idx != P_MAX_INDEX) {
           substitution = ServiceMacros[idx].Translate(request, args, text(startpos, endpos-1));
           substitedMacro = PTrue;
         }
@@ -1681,6 +1677,25 @@ PBoolean PServiceHTML::ProcessMacros(PHTTPRequest & request,
 }
 
 
+bool PServiceHTML::SpliceMacro(PString & text, const PString & tokens, const PString & value)
+{
+  PString adjustedTokens = tokens;
+  adjustedTokens.Replace(" ", "[ \t\r\n]+");
+  PRegularExpression RegEx("<?!--#" + adjustedTokens + "[ \t\r\n]*-->?",
+                           PRegularExpression::Extended|PRegularExpression::IgnoreCase);
+
+  bool foundOne = false;
+
+  PINDEX pos, len;
+  while (text.FindRegEx(RegEx, pos, len)) {
+    text.Splice(value, pos, len);
+    foundOne = true;
+  }
+
+  return foundOne;
+}
+
+
 ///////////////////////////////////////////////////////////////////
 
 static void ServiceOnLoadedText(PString & text)
