@@ -23,9 +23,9 @@
  *
  * Contributor(s): Federico Pinna and Reitek S.p.A. (SASL authentication)
  *
- * $Revision: 21788 $
+ * $Revision: 26045 $
  * $Author: rjongbloed $
- * $Date: 2008-12-11 23:42:13 -0600 (Thu, 11 Dec 2008) $
+ * $Date: 2011-06-19 20:28:39 -0500 (Sun, 19 Jun 2011) $
  */
 
 #ifdef __GNUC__
@@ -35,12 +35,12 @@
 #include <ptlib.h>
 #include <ptlib/sockets.h>
 #include <ptclib/inetmail.h>
-#if P_SASL2
+#if P_SASL
 #include <ptclib/psasl.h>
 #endif
 
-static const PString CRLF = "\r\n";
-static const PString CRLFdotCRLF = "\r\n.\r\n";
+static const PConstString CRLF("\r\n");
+static const PConstString CRLFdotCRLF("\r\n.\r\n");
 
 
 #define new PNEW
@@ -101,7 +101,7 @@ PBoolean PSMTPClient::Close()
 }
 
 
-#if P_SASL2
+#if P_SASL
 PBoolean PSMTPClient::LogIn(const PString & username,
                         const PString & password)
 {
@@ -223,7 +223,7 @@ PBoolean PSMTPClient::BeginMessage(const PString & from,
 }
 
 
-PBoolean PSMTPClient::InternalBeginMessage()
+bool PSMTPClient::InternalBeginMessage()
 {
   PString localHost;
   PString peerHost;
@@ -810,8 +810,8 @@ static char const * const POP3Commands[PPOP3::NumCommands] = {
 };
 
 
-PString PPOP3::okResponse = "+OK";
-PString PPOP3::errResponse = "-ERR";
+const PString & PPOP3::okResponse () { static const PConstString s("+OK");  return s; }
+const PString & PPOP3::errResponse() { static const PConstString s("-ERR"); return s; }
 
 
 PPOP3::PPOP3()
@@ -874,7 +874,7 @@ PBoolean PPOP3Client::Close()
 
 PBoolean PPOP3Client::LogIn(const PString & username, const PString & password, int options)
 {
-#if P_SASL2
+#if P_SASL
   PString mech;
   PSASLClient auth("pop", username, username, password);
 
@@ -924,9 +924,9 @@ PBoolean PPOP3Client::LogIn(const PString & username, const PString & password,
     } while (result == PSASLClient::Continue);
     auth.End();
   }
-  else {
+  else
 #endif
-
+  {
     if (!apopBanner.IsEmpty()) { // let's try with APOP
 
       PMessageDigest::Result bin_digest;
@@ -953,10 +953,7 @@ PBoolean PPOP3Client::LogIn(const PString & username, const PString & password,
 
     if (ExecuteCommand(PASS, password) <= 0)
       return PFalse;
-
-#if P_SASL2
   }
-#endif
 
   loggedIn = PTrue;
   return PTrue;
@@ -1025,7 +1022,7 @@ PPOP3Server::PPOP3Server()
 
 PBoolean PPOP3Server::OnOpen()
 {
-  return WriteResponse(okResponse, PIPSocket::GetHostName() +
+  return WriteResponse(okResponse(), PIPSocket::GetHostName() +
                      " POP3 server ready at " +
                       PTime(PTime::MediumDateTime).AsString());
 }
@@ -1068,7 +1065,7 @@ PBoolean PPOP3Server::ProcessCommand()
       break;
     case TOP :
       if (args.Find(' ') == P_MAX_INDEX)
-        WriteResponse(errResponse, "Syntax error");
+        WriteResponse(errResponse(), "Syntax error");
       else
         OnTOP((PINDEX)args.AsInteger(),
               (PINDEX)args.Mid(args.Find(' ')).AsInteger());
@@ -1089,18 +1086,18 @@ void PPOP3Server::OnUSER(const PString & name)
   messageSizes.SetSize(0);
   messageIDs.SetSize(0);
   username = name;
-  WriteResponse(okResponse, "User name accepted.");
+  WriteResponse(okResponse(), "User name accepted.");
 }
 
 
 void PPOP3Server::OnPASS(const PString & password)
 {
   if (username.IsEmpty())
-    WriteResponse(errResponse, "No user name specified.");
+    WriteResponse(errResponse(), "No user name specified.");
   else if (HandleOpenMailbox(username, password))
-    WriteResponse(okResponse, username + " mail is available.");
+    WriteResponse(okResponse(), username + " mail is available.");
   else
-    WriteResponse(errResponse, "No access to " + username + " mail.");
+    WriteResponse(errResponse(), "No access to " + username + " mail.");
   messageDeletions.SetSize(messageIDs.GetSize());
 }
 
@@ -1111,7 +1108,7 @@ void PPOP3Server::OnQUIT()
     if (messageDeletions[i])
       HandleDeleteMessage(i+1, messageIDs[i]);
 
-  WriteResponse(okResponse, PIPSocket::GetHostName() +
+  WriteResponse(okResponse(), PIPSocket::GetHostName() +
                      " POP3 server signing off at " +
                       PTime(PTime::MediumDateTime).AsString());
 
@@ -1123,13 +1120,13 @@ void PPOP3Server::OnRSET()
 {
   for (PINDEX i = 0; i < messageDeletions.GetSize(); i++)
     messageDeletions[i] = PFalse;
-  WriteResponse(okResponse, "Resetting state.");
+  WriteResponse(okResponse(), "Resetting state.");
 }
 
 
 void PPOP3Server::OnNOOP()
 {
-  WriteResponse(okResponse, "Doing nothing.");
+  WriteResponse(okResponse(), "Doing nothing.");
 }
 
 
@@ -1138,32 +1135,32 @@ void PPOP3Server::OnSTAT()
   DWORD total = 0;
   for (PINDEX i = 0; i < messageSizes.GetSize(); i++)
     total += messageSizes[i];
-  WriteResponse(okResponse, psprintf("%u %u", messageSizes.GetSize(), total));
+  WriteResponse(okResponse(), psprintf("%u %u", messageSizes.GetSize(), total));
 }
 
 
 void PPOP3Server::OnLIST(PINDEX msg)
 {
   if (msg == 0) {
-    WriteResponse(okResponse, psprintf("%u messages.", messageSizes.GetSize()));
+    WriteResponse(okResponse(), psprintf("%u messages.", messageSizes.GetSize()));
     for (PINDEX i = 0; i < messageSizes.GetSize(); i++)
       if (!messageDeletions[i])
         WriteLine(psprintf("%u %u", i+1, messageSizes[i]));
     WriteLine(".");
   }
   else if (msg < 1 || msg > messageSizes.GetSize())
-    WriteResponse(errResponse, "No such message.");
+    WriteResponse(errResponse(), "No such message.");
   else
-    WriteResponse(okResponse, psprintf("%u %u", msg, messageSizes[msg-1]));
+    WriteResponse(okResponse(), psprintf("%u %u", msg, messageSizes[msg-1]));
 }
 
 
 void PPOP3Server::OnRETR(PINDEX msg)
 {
   if (msg < 1 || msg > messageDeletions.GetSize())
-    WriteResponse(errResponse, "No such message.");
+    WriteResponse(errResponse(), "No such message.");
   else {
-    WriteResponse(okResponse,
+    WriteResponse(okResponse(),
                  PString(PString::Unsigned, messageSizes[msg-1]) + " octets.");
     stuffingState = StuffIdle;
     HandleSendMessage(msg, messageIDs[msg-1], P_MAX_INDEX);
@@ -1176,10 +1173,10 @@ void PPOP3Server::OnRETR(PINDEX msg)
 void PPOP3Server::OnDELE(PINDEX msg)
 {
   if (msg < 1 || msg > messageDeletions.GetSize())
-    WriteResponse(errResponse, "No such message.");
+    WriteResponse(errResponse(), "No such message.");
   else {
     messageDeletions[msg-1] = PTrue;
-    WriteResponse(okResponse, "Message marked for deletion.");
+    WriteResponse(okResponse(), "Message marked for deletion.");
   }
 }
 
@@ -1187,9 +1184,9 @@ void PPOP3Server::OnDELE(PINDEX msg)
 void PPOP3Server::OnTOP(PINDEX msg, PINDEX count)
 {
   if (msg < 1 || msg > messageDeletions.GetSize())
-    WriteResponse(errResponse, "No such message.");
+    WriteResponse(errResponse(), "No such message.");
   else {
-    WriteResponse(okResponse, "Top of message");
+    WriteResponse(okResponse(), "Top of message");
     stuffingState = StuffIdle;
     HandleSendMessage(msg, messageIDs[msg-1], count);
     stuffingState = DontStuff;
@@ -1201,7 +1198,7 @@ void PPOP3Server::OnTOP(PINDEX msg, PINDEX count)
 void PPOP3Server::OnUIDL(PINDEX msg)
 {
   if (msg == 0) {
-    WriteResponse(okResponse,
+    WriteResponse(okResponse(),
               PString(PString::Unsigned, messageIDs.GetSize()) + " messages.");
     for (PINDEX i = 0; i < messageIDs.GetSize(); i++)
       if (!messageDeletions[i])
@@ -1209,7 +1206,7 @@ void PPOP3Server::OnUIDL(PINDEX msg)
     WriteLine(".");
   }
   else if (msg < 1 || msg > messageSizes.GetSize())
-    WriteResponse(errResponse, "No such message.");
+    WriteResponse(errResponse(), "No such message.");
   else
     WriteLine(PString(PString::Unsigned, msg) & messageIDs[msg-1]);
 }
@@ -1217,7 +1214,7 @@ void PPOP3Server::OnUIDL(PINDEX msg)
 
 PBoolean PPOP3Server::OnUnknown(const PCaselessString & command)
 {
-  WriteResponse(errResponse, "Command \"" + command + "\" unrecognised.");
+  WriteResponse(errResponse(), "Command \"" + command + "\" unrecognised.");
   return PTrue;
 }
 
@@ -1241,20 +1238,17 @@ void PPOP3Server::HandleDeleteMessage(PINDEX, const PString &)
 //////////////////////////////////////////////////////////////////////////////
 // PRFC822Channel
 
-const PString & PRFC822Channel::MimeVersionTag() { static PString s = "MIME-version"; return s; }
-const PString & PRFC822Channel::FromTag() { static PString s = "From"; return s; }
-const PString & PRFC822Channel::ToTag() { static PString s = "To"; return s; }
-const PString & PRFC822Channel::CCTag() { static PString s = "cc"; return s; }
-const PString & PRFC822Channel::BCCTag() { static PString s = "bcc"; return s; }
-const PString & PRFC822Channel::SubjectTag() { static PString s = "Subject"; return s; }
-const PString & PRFC822Channel::DateTag() { static PString s = "Date"; return s; }
-const PString & PRFC822Channel::ReturnPathTag() { static PString s = "Return-Path"; return s; }
-const PString & PRFC822Channel::ReceivedTag() { static PString s = "Received"; return s; }
-const PString & PRFC822Channel::MessageIDTag() { static PString s = "Message-ID"; return s; }
-const PString & PRFC822Channel::MailerTag() { static PString s = "X-Mailer"; return s; }
-const PString & PRFC822Channel::ContentTypeTag() { static PString s = "Content-Type"; return s; }
-const PString & PRFC822Channel::ContentDispositionTag() { static PString s = "Content-Disposition"; return s; }
-const PString & PRFC822Channel::ContentTransferEncodingTag() { static PString s = "Content-Transfer-Encoding"; return s; }
+const PCaselessString & PRFC822Channel::MimeVersionTag() { static const PConstCaselessString s("MIME-version"); return s; }
+const PCaselessString & PRFC822Channel::FromTag()        { static const PConstCaselessString s("From");         return s; }
+const PCaselessString & PRFC822Channel::ToTag()          { static const PConstCaselessString s("To");           return s; }
+const PCaselessString & PRFC822Channel::CCTag()          { static const PConstCaselessString s("cc");           return s; }
+const PCaselessString & PRFC822Channel::BCCTag()         { static const PConstCaselessString s("bcc");          return s; }
+const PCaselessString & PRFC822Channel::SubjectTag()     { static const PConstCaselessString s("Subject");      return s; }
+const PCaselessString & PRFC822Channel::DateTag()        { static const PConstCaselessString s("Date");         return s; }
+const PCaselessString & PRFC822Channel::ReturnPathTag()  { static const PConstCaselessString s("Return-Path");  return s; }
+const PCaselessString & PRFC822Channel::ReceivedTag()    { static const PConstCaselessString s("Received");     return s; }
+const PCaselessString & PRFC822Channel::MessageIDTag()   { static const PConstCaselessString s("Message-ID");   return s; }
+const PCaselessString & PRFC822Channel::MailerTag()      { static const PConstCaselessString s("X-Mailer");     return s; }
 
 
 
@@ -1298,7 +1292,7 @@ PBoolean PRFC822Channel::Write(const void * buf, PINDEX len)
     if (writePartHeaders)
       headers.SetAt(ContentTypeTag(), "multipart/mixed; boundary=\""+boundaries.front()+'"');
     else if (!headers.Contains(ContentTypeTag()))
-      headers.SetAt(ContentTypeTag(), "text/plain");
+      headers.SetAt(ContentTypeTag(), PMIMEInfo::TextPlain());
 
     PStringStream hdr;
     hdr << ::setfill('\r') << headers;
@@ -1313,7 +1307,7 @@ PBoolean PRFC822Channel::Write(const void * buf, PINDEX len)
 
   if (writePartHeaders) {
     if (!partHeaders.Contains(ContentTypeTag()))
-      partHeaders.SetAt(ContentTypeTag(), "text/plain");
+      partHeaders.SetAt(ContentTypeTag(), PMIMEInfo::TextPlain());
 
     PStringStream hdr;
     hdr << "\n--"  << boundaries.front() << '\n'
@@ -1374,7 +1368,7 @@ PString PRFC822Channel::MultipartMessage()
   PString boundary;
 
   do {
-    boundary.sprintf("PWLib.%lu.%u", time(NULL), rand());
+    boundary.sprintf("PWLib.%lu.%u", PTime().GetTimeInSeconds(), rand());
   } while (!MultipartMessage(boundary));
 
   return boundary;
