@@ -23,9 +23,9 @@
  *
  * Copyright 2003 Equivalence Pty. Ltd.
  *
- * $Revision: 25959 $
+ * $Revision: 28568 $
  * $Author: rjongbloed $
- * $Date: 2011-06-02 00:33:14 -0500 (Thu, 02 Jun 2011) $
+ * $Date: 2012-11-22 00:16:50 -0600 (Thu, 22 Nov 2012) $
  */
 
 #ifdef __GNUC__
@@ -39,45 +39,24 @@
 
 #define new PNEW
 
-#define USE_RESOLVER_CACHING    1
 #define RESOLVER_CACHE_TIMEOUT  30000
 
 #if P_DNS
 
-#ifdef _MSC_VER
+#ifdef _WIN32
   #pragma comment(lib, "DnsAPI.Lib")
   #pragma message("DNS support enabled")
-#else
-  enum { DnsFreeFlat = 0 };
 #endif
 
 
 /////////////////////////////////////////////////
 
-#if defined(P_HAS_RESOLVER) ||  defined(USE_RESOLVER_CACHING)
-
-void PDnsRecordListFree(PDNS_RECORD rec, int /* FreeType */)
-{
-  while (rec != NULL) {
-    PDNS_RECORD next = rec->pNext;
-    free(rec);
-    rec = next;
-  }
-}
-
-#endif
-
-#if defined(USE_RESOLVER_CACHING) || (defined(P_HAS_RESOLVER) && !defined(P_HAS_RES_NINIT))
-
 static PMutex & GetDNSMutex()
 {
   static PMutex mutex;
   return mutex;
 }
 
-#endif
-
-#if defined(USE_RESOLVER_CACHING) 
 
 struct DNSCacheInfo {
   DNSCacheInfo() : m_results(NULL), m_status(-1) { }
@@ -91,7 +70,7 @@ typedef std::map<std::string, DNSCacheInfo> DNSCache;
 static PTime g_lastAgeTime(0);
 static DNSCache g_dnsCache;
 
-#endif
+
 
 #ifdef P_HAS_RESOLVER
 
@@ -186,6 +165,15 @@ static PBoolean ProcessDNSRecords(
         GETLONG(newRecord->Data.A.IpAddress, data);
         break;
 
+      case T_AAAA:
+        newRecord = (PDNS_RECORD)malloc(sizeof(DnsRecord)); 
+        memset(newRecord, 0, sizeof(DnsRecord));
+        GETLONG(newRecord->Data.AAAA.Ip6Address[0], data);
+        GETLONG(newRecord->Data.AAAA.Ip6Address[1], data);
+        GETLONG(newRecord->Data.AAAA.Ip6Address[2], data);
+        GETLONG(newRecord->Data.AAAA.Ip6Address[3], data);
+        break;
+
       case T_NS:
         newRecord = (PDNS_RECORD)malloc(sizeof(DnsRecord)); 
         memset(newRecord, 0, sizeof(DnsRecord));
@@ -220,7 +208,7 @@ static PBoolean ProcessDNSRecords(
 DNS_STATUS DnsQuery_A(const char * service,
                               WORD requestType,
                              DWORD options,
-                            PIP4_ARRAY,
+                            void *,
                      PDNS_RECORD * results,
                             void *)
 {
@@ -285,7 +273,7 @@ DNS_STATUS DnsQuery_A(const char * service,
        ntohs(reply.hdr.nscount),
        ntohs(reply.hdr.arcount),
        results)) {
-    PDnsRecordListFree(*results, 0);
+    DnsRecordListFree(*results, DnsFreeRecordList);
     return -1;
   }
 
@@ -293,8 +281,41 @@ DNS_STATUS DnsQuery_A(const char * service,
 }
 
 
+PDNS_RECORD DnsRecordSetCopy(PDNS_RECORD src)
+{
+  PDNS_RECORD result = NULL;
+  PDNS_RECORD dst = NULL;
+  PDNS_RECORD rec;
+
+  while (src != NULL) {
+    rec = (PDNS_RECORD)malloc(sizeof(DNS_RECORD));
+    memcpy(rec, src, sizeof(DNS_RECORD));
+    if (result == NULL)
+      result = rec;
+    rec->pNext = NULL;
+    if (dst != NULL)
+      dst->pNext = rec;
+    src = src->pNext;
+    dst = rec;
+  }
+
+  return result;
+}
+
+
+void DnsRecordListFree(PDNS_RECORD rec, int /* FreeType */)
+{
+  while (rec != NULL) {
+    PDNS_RECORD next = rec->pNext;
+    free(rec);
+    rec = next;
+  }
+}
+
+
 #endif // P_HAS_RESOLVER
 
+
 PObject::Comparison PDNS::SRVRecord::Compare(const PObject & obj) const
 {
   const SRVRecord * other = dynamic_cast<const SRVRecord *>(&obj);
@@ -345,17 +366,21 @@ PDNS::SRVRecord * PDNS::SRVRecordList::HandleDNSRecord(PDNS_RECORD dnsRecord, PD
     record->priority = dnsRecord->Data.SRV.wPriority;
     record->weight   = dnsRecord->Data.SRV.wWeight;
 
-    // see if any A records match this hostname
+    // see if any A or AAAA records match this hostname
     PDNS_RECORD aRecord = results;
     while (aRecord != NULL) {
       if ((dnsRecord->Flags.S.Section == DnsSectionAdditional) && (dnsRecord->wType == DNS_TYPE_A)) {
         record->hostAddress = PIPSocket::Address(dnsRecord->Data.A.IpAddress);
         break;
       }
+      if ((dnsRecord->Flags.S.Section == DnsSectionAdditional) && (dnsRecord->wType == DNS_TYPE_AAAA)) {
+        record->hostAddress = PIPSocket::Address(16, (BYTE *)&dnsRecord->Data.AAAA.Ip6Address);
+        break;
+      }
       aRecord = aRecord->pNext;
     }
 
-    // if no A record found, then get address the hard way
+    // if no A or AAAA record found, then get address the hard way
     if (aRecord == NULL)
       PIPSocket::GetHostAddress(record->hostName, record->hostAddress);
   }
@@ -506,8 +531,12 @@ PBoolean PDNS::LookupSRV(
     user = user + "@";
 
   PIPSocketAddressAndPortVector::const_iterator r;
-  for (r = info.begin(); r != info.end(); ++r) 
-    returnList.AppendString(user + r->AsString(':'));
+  for (r = info.begin(); r != info.end(); ++r) {
+    if (r->GetAddress().GetVersion() == 6)
+      returnList.AppendString(user + "[" + r->GetAddress().AsString() + "]:" + PString(r->GetPort()));
+    else
+      returnList.AppendString(user + r->AsString(':'));
+  }
 
   return returnList.GetSize() != 0;;
 }
@@ -524,13 +553,12 @@ PBoolean PDNS::LookupSRV(
     return PFalse;
   }
 
-  PTRACE(4,"DNS\tSRV Lookup " << domain << " service " << service);
-  
   PString srvLookupStr = service;
   if (srvLookupStr.Right(1) != ".")
     srvLookupStr += ".";
   srvLookupStr += domain;
   
+  PTRACE(4,"DNS\tSRV Lookup \"" << srvLookupStr << '"');
   return LookupSRV(srvLookupStr, defaultPort, addrList);
 }
 
@@ -544,10 +572,11 @@ PBoolean PDNS::LookupSRV(
   PDNS::SRVRecordList srvRecords;
   PBoolean found = PDNS::GetRecords(srvLookupStr, srvRecords);
   if (found) {
-    PTRACE(5,"DNS\tSRV Record found " << srvLookupStr);
+    PTRACE(5,"DNS\tSRV Record found \"" << srvLookupStr << '"');
     PDNS::SRVRecord * recPtr = srvRecords.GetFirst();
     while (recPtr != NULL) {
-      PIPSocketAddressAndPort addrAndPort(recPtr->hostAddress, recPtr->port > 0 ? recPtr->port : defaultPort);
+      PIPSocketAddressAndPort addrAndPort;
+      addrAndPort.SetAddress(recPtr->hostAddress, recPtr->port > 0 ? recPtr->port : defaultPort);
       addrList.push_back(addrAndPort);
 
       recPtr = srvRecords.GetNext();
@@ -605,6 +634,10 @@ PDNS::MXRecord * PDNS::MXRecordList::HandleDNSRecord(PDNS_RECORD dnsRecord, PDNS
         record->hostAddress = PIPSocket::Address(dnsRecord->Data.A.IpAddress);
         break;
       }
+      if ((dnsRecord->Flags.S.Section == DnsSectionAdditional) && (dnsRecord->wType == DNS_TYPE_AAAA)) {
+        record->hostAddress = PIPSocket::Address(16, (BYTE *)&dnsRecord->Data.AAAA.Ip6Address);
+        break;
+      }
       aRecord = aRecord->pNext;
     }
 
@@ -647,52 +680,6 @@ PDNS::MXRecord * PDNS::MXRecordList::GetNext()
 
 /////////////////////////////////////////////////////////////////
 
-#ifndef USE_RESOLVER_CACHING
-
-DNS_STATUS PDNS::Cached_DnsQuery(
-    const char * name,
-    WORD       type,
-    DWORD      ,
-    void *     ,
-    PDNS_RECORD * queryResults,
-    void * )
-{
-  return DnsQuery_A((const char *)name, 
-                               type,
-                               DNS_QUERY_STANDARD, 
-                               (PIP4_ARRAY)NULL, 
-                               queryResults, 
-                               NULL);
-}
-
-void PDNS::Cached_DnsRecordListFree(PDNS_RECORD results, int)
-{
-  DnsRecordListFree(results, 0);
-}
-
-#else  // ifndef USE_RESOLVER_CACHING
-
-static PDNS_RECORD DnsRecordListClone(PDNS_RECORD src)
-{
-  PDNS_RECORD result = NULL;
-  PDNS_RECORD dst = NULL;
-  PDNS_RECORD rec;
-
-  while (src != NULL) {
-    rec = (PDNS_RECORD)malloc(sizeof(DNS_RECORD));
-    memcpy(rec, src, sizeof(DNS_RECORD));
-    if (result == NULL)
-      result = rec;
-    rec->pNext = NULL;
-    if (dst != NULL)
-      dst->pNext = rec;
-    src = src->pNext;
-    dst = rec;
-  }
-
-  return result;
-}
-
 DNS_STATUS PDNS::Cached_DnsQuery(
     const char * name,
     WORD       type,
@@ -714,43 +701,53 @@ DNS_STATUS PDNS::Cached_DnsQuery(
     while (r != g_dnsCache.end()) {
       if ((now - r->second.m_time) < RESOLVER_CACHE_TIMEOUT)
         ++r;
-      else { 
-        DnsRecordListFree(r->second.m_results, DnsFreeFlat);
+      else {
+        PTRACE(5, "DNS\tQuery aged \"" << r->first << '"');
+        DnsRecordListFree(r->second.m_results, DnsFreeRecordList);
         g_dnsCache.erase(r++);
       }
     }
   }
 
   // see if cache contains the entry we need
-  std::stringstream key;
-  key << name << '\n' << type << '\n' << options;
+  string key;
+  {
+    std::stringstream strm;
+    strm << name << '\t' << type << '\t' << options;
+    key = strm.str();
+  }
 
-  r = g_dnsCache.find(key.str());
+  r = g_dnsCache.find(key);
   if (r == g_dnsCache.end()) {
+    PTRACE(5, "DNS\tSRV physical lookup \"" << key << '"');
 
     // else do the lookup and put it into the cache
     DNSCacheInfo info;
     info.m_status = DnsQuery_A((const char *)name, 
                                type,
                                DNS_QUERY_STANDARD, 
-                               (PIP4_ARRAY)NULL, 
+                               NULL, 
                                &info.m_results, 
                                NULL);
+#if PTRACING
+    if (info.m_status != 0)
+      PTRACE(3, "DNS\tQuery failed: error=" << info.m_status);
+    else {
+      PTRACE(6, "DNS\tQuery success: " << info.m_results);
+      for (PDNS_RECORD rec = info.m_results; rec != NULL; rec = rec->pNext)
+        PTRACE(6, "DNS\tQuery: name=\"" << PString(rec->pName)
+               << "\", type=" << rec->wType << ", len=" << rec->wDataLength);
+      PTRACE(6, "DNS\tQuery done");
+    }
+#endif
 
-    r = g_dnsCache.insert(DNSCache::value_type(key.str(), info)).first;
+    r = g_dnsCache.insert(DNSCache::value_type(key, info)).first;
   }
 
-  *queryResults = DnsRecordListClone(r->second.m_results);
+  *queryResults = DnsRecordSetCopy(r->second.m_results);
   return r->second.m_status;
 }
 
-void PDNS::Cached_DnsRecordListFree(PDNS_RECORD results, int)
-{
-  PDnsRecordListFree(results, 0);
-}
-
-
-#endif  // ifndef USE_RESOLVER_CACHING
 
 /////////////////////////////////////////////////////////////////
 
