@@ -27,9 +27,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22249 $
- * $Author: rjongbloed $
- * $Date: 2009-03-22 22:59:45 -0500 (Sun, 22 Mar 2009) $
+ * $Revision: 24971 $
+ * $Author: willamowius $
+ * $Date: 2010-12-21 05:08:30 -0600 (Tue, 21 Dec 2010) $
  */
 
 #include <ptlib.h>
@@ -679,7 +679,7 @@ PBoolean PODBC::Field::DataFragment(PString & Buffer,PINDEX & fragment, SQLINTEG
 // Buffer Fragment
   if (fragcount < fragment) {
 	  Buffer.Splice(Data.sbinlong,(fragment * size)+1,size);
-	  fragment = fragment++;
+	  fragment++;
 	  return PTrue;
   }
 // Last Fragment
