@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23101 $
+ * $Revision: 26549 $
  * $Author: rjongbloed $
- * $Date: 2009-07-13 01:30:19 -0500 (Mon, 13 Jul 2009) $
+ * $Date: 2011-10-05 23:24:38 -0500 (Wed, 05 Oct 2011) $
  */
 
 #ifdef __GNUC__
@@ -49,6 +49,9 @@
 typedef PSTUNClient PNatMethod_STUN;
 PCREATE_NAT_PLUGIN(STUN);
 
+PFACTORY_CREATE(PFactory<PNatMethod>, PSTUNClient, "STUN");
+
+
 ///////////////////////////////////////////////////////////////////////
 
 PSTUNClient::PSTUNClient()
@@ -416,19 +419,25 @@ public:
   
   bool Write(PUDPSocket & socket) const
   {
-    return socket.Write(theArray, ((PSTUNMessageHeader *)theArray)->msgLength+sizeof(PSTUNMessageHeader)) != PFalse;
+    if (socket.Write(theArray, ((PSTUNMessageHeader *)theArray)->msgLength+sizeof(PSTUNMessageHeader)))
+      return true;
+
+    PTRACE(1, "STUN\tError writing to " << socket.GetSendAddress()
+           << " - " << socket.GetErrorText(PChannel::LastWriteError));
+    return false;
   }
 
   bool Poll(PUDPSocket & socket, const PSTUNMessage & request, PINDEX pollRetries)
   {
     for (PINDEX retry = 0; retry < pollRetries; retry++) {
       if (!request.Write(socket))
-        break;
+        return false;
 
       if (Read(socket) && Validate(request))
         return true;
     }
 
+    PTRACE(5, "STUN\tNo response from " << socket.GetSendAddress() << " after " << pollRetries << " retries.");
     return false;
   }
 };
@@ -478,9 +487,9 @@ PSTUNClient::NatTypes PSTUNClient::GetNatType(PBoolean force)
 
   PIPSocket::InterfaceTable interfaces;
   if (PIPSocket::GetInterfaceTable(interfaces)) {
-    for (PINDEX i =0; i < interfaces.GetSize(); i++) {
+    for (PINDEX i = 0; i < interfaces.GetSize(); i++) {
       PIPSocket::Address binding = interfaces[i].GetAddress();
-      if (!binding.IsLoopback()) {
+      if (!binding.IsLoopback() && binding.GetVersion() == 4) {
         PUDPSocket * socket = new PUDPSocket;
         if (OpenSocket(*socket, singlePortInfo, binding))
           sockets.Append(socket);
@@ -517,9 +526,6 @@ PSTUNClient::NatTypes PSTUNClient::GetNatType(PBoolean force)
     for (PList<PUDPSocket>::iterator socket = sockets.begin(); socket != sockets.end(); ++socket) {
       if (requestI.Write(*socket))
         selectList += *socket;
-      else {
-        PTRACE(1, "STUN\tError writing to " << *this << " - " << socket->GetErrorText(PChannel::LastWriteError));
-      }
     }
 
     if (selectList.IsEmpty())
@@ -569,6 +575,9 @@ PSTUNClient::NatTypes PSTUNClient::GetNatType(PBoolean force)
     return natType = (testII ? OpenNat : SymmetricFirewall);
   }
 
+  cachedExternalAddress = mappedAddressI;
+  timeAddressObtained.SetCurrentTime();
+
   if (testII)
     return natType = ConeNat;
 
@@ -671,7 +680,7 @@ PBoolean PSTUNClient::GetExternalAddress(PIPSocket::Address & externalAddress,
   PSTUNMessage response;
   if (!response.Poll(socket, request, pollRetries))
   {
-    PTRACE(1, "STUN\t" << *this << " unexpectedly went offline.");
+    PTRACE(1, "STUN\t" << *this << " unexpectedly went offline getting external address.");
     return false;
   }
 
@@ -684,7 +693,7 @@ PBoolean PSTUNClient::GetExternalAddress(PIPSocket::Address & externalAddress,
 
   
   externalAddress = cachedExternalAddress = mappedAddress->GetIP();
-  timeAddressObtained = PTime();
+  timeAddressObtained.SetCurrentTime();
   return true;
 }
 
@@ -771,7 +780,7 @@ PBoolean PSTUNClient::CreateSocket(PUDPSocket * & socket, const PIPSocket::Addre
       PTRACE(2, "STUN\tExpected mapped address attribute from " << *this);
     }
     else
-      PTRACE(1, "STUN\t" << *this << " unexpectedly went offline.");
+      PTRACE(1, "STUN\t" << *this << " unexpectedly went offline creating socket.");
   }
 
   delete stunSocket;
@@ -836,7 +845,7 @@ PBoolean PSTUNClient::CreateSocketPair(PUDPSocket * & socket1,
   {
     if (!response[i].Poll(stunSocket[i], request[i], pollRetries))
     {
-      PTRACE(1, "STUN\t" << *this << " unexpectedly went offline.");
+      PTRACE(1, "STUN\t" << *this << " unexpectedly went offline creating socket pair.");
       return false;
     }
   }
