@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22937 $
+ * $Revision: 27468 $
  * $Author: rjongbloed $
- * $Date: 2009-06-21 07:41:37 -0500 (Sun, 21 Jun 2009) $
+ * $Date: 2012-04-18 14:31:01 -0500 (Wed, 18 Apr 2012) $
  */
 
 #ifdef __GNUC__
@@ -85,6 +85,7 @@ PSoundChannel_WAVFile::PSoundChannel_WAVFile()
   : m_autoRepeat(false)
   , m_sampleRate(8000)
   , m_bufferSize(2)
+  , m_samplePosition(P_MAX_INDEX)
 {
 }
 
@@ -264,7 +265,7 @@ PBoolean PSoundChannel_WAVFile::Read(void * data, PINDEX size)
       iDutyCycle += wavSampleRate;
       if (iDutyCycle >= m_sampleRate) {
         iDutyCycle -= m_sampleRate;
-        if (!ReadSamples(&iSample, sizeof(short)))
+        if (!ReadSample(iSample))
           return false;
       }
       *pPCM++ = iSample;
@@ -278,7 +279,7 @@ PBoolean PSoundChannel_WAVFile::Read(void * data, PINDEX size)
     short * pPCM = (short *)data;
     for (PINDEX count = 0; count < size; count += sizeof(short)) {
       do {
-        if (!ReadSamples(&iSample, sizeof(short)))
+        if (!ReadSample(iSample))
           return false;
         iDutyCycle += m_sampleRate;
       } while (iDutyCycle < wavSampleRate);
@@ -298,9 +299,23 @@ PBoolean PSoundChannel_WAVFile::Read(void * data, PINDEX size)
 }
 
 
+bool PSoundChannel_WAVFile::ReadSample(short & sample)
+{
+  if (m_samplePosition >= m_sampleBuffer.GetSize()) {
+    static const PINDEX BufferSize = 10000;
+    if (!ReadSamples(m_sampleBuffer.GetPointer(BufferSize), BufferSize*sizeof(short)))
+      return false;
+    m_sampleBuffer.SetSize(m_WAVFile.GetLastReadCount()/sizeof(short));
+    m_samplePosition = 0;
+  }
+  sample = m_sampleBuffer[m_samplePosition++];
+  return true;
+}
+
+
 bool PSoundChannel_WAVFile::ReadSamples(void * data, PINDEX size)
 {
-  if (m_WAVFile.Read(data, size) && m_WAVFile.GetLastReadCount() >= size)
+  if (m_WAVFile.Read(data, size))
     return true;
 
   if (!m_autoRepeat)
