@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25124 $
- * $Author: rjongbloed $
- * $Date: 2011-02-16 18:34:29 -0600 (Wed, 16 Feb 2011) $
+ * $Revision: 28856 $
+ * $Author: csoutheren $
+ * $Date: 2013-01-11 19:09:35 -0600 (Fri, 11 Jan 2013) $
  */
 
 // This depends on the expat XML library by Jim Clark
@@ -57,6 +57,15 @@
 #endif
 
 
+////////////////////////////////////////////////////
+
+PXMLBase::PXMLBase(int opts)
+  : m_options(opts)
+  , m_maxEntityLength(PXML::DEFAULT_MAX_ENTITY_LENGTH)
+{
+}
+
+
 ////////////////////////////////////////////////////
 
 static void PXML_StartElement(void * userData, const char * name, const char ** attrs)
@@ -198,6 +207,13 @@ void PXMLParser::EndElement(const char * /*name*/)
 
 void PXMLParser::AddCharacterData(const char * data, int len)
 {
+  unsigned checkLen = len + ((lastElement != NULL) ? lastElement->GetString().GetLength() : 0);
+  if (checkLen >= m_maxEntityLength) {
+    PTRACE(2, "PXML\tAborting XML parse at size " << m_maxEntityLength << " - possible 'billion laugh' attack");
+    XML_StopParser((XML_Parser)expat, XML_FALSE);
+    return;
+  }
+
   PString str(data, len);
 
   if (lastElement != NULL) {
@@ -260,9 +276,6 @@ PXML::PXML(const PString & data, int options, const char * noIndentElements)
 
 PXML::~PXML()
 {
-#if P_HTTP
-  autoLoadTimer.Stop();
-#endif // P_HTTP
   RemoveAll();
 }
 
@@ -368,140 +381,6 @@ bool PXML::LoadFile(const PFilePath & fn, PXMLParser::Options options)
 }
 
 
-#if P_HTTP
-
-bool PXML::LoadURL(const PURL & url)
-{
-  return LoadURL(url, PMaxTimeInterval, PXMLParser::NoOptions);
-}
-
-
-bool PXML::LoadURL(const PURL & url, const PTimeInterval & timeout, PXMLParser::Options options)
-{
-  if (url.IsEmpty()) {
-    m_errorString = "Cannot load empty URL";
-    m_errorLine = m_errorColumn = 0;
-    return false;
-  }
-
-  PTRACE(4, "XML\tLoading URL " << url);
-
-  PString data;
-  if (url.GetScheme() == "file") 
-    return LoadFile(url.AsFilePath());
-
-  PHTTPClient client;
-  PINDEX contentLength;
-  PMIMEInfo outMIME, replyMIME;
-
-  // make sure we do not hang around for ever
-  client.SetReadTimeout(timeout);
-
-  // get the resource header information
-  if (!client.GetDocument(url, outMIME, replyMIME)) {
-    m_errorString = "Cannot load URL ";
-    m_errorLine = m_errorColumn = 0;
-    m_errorString << '"' << url << '"';
-    return false;
-  }
-
-  // get the length of the data
-  if (replyMIME.Contains(PHTTPClient::ContentLengthTag()))
-    contentLength = (PINDEX)replyMIME[PHTTPClient::ContentLengthTag()].AsUnsigned();
-  else
-    contentLength = P_MAX_INDEX;
-
-  // download the resource into memory
-  PINDEX offs = 0;
-  for (;;) {
-    PINDEX len;
-    if (contentLength == P_MAX_INDEX)
-      len = CACHE_BUFFER_SIZE;
-    else if (offs == contentLength)
-      break;
-    else
-      len = PMIN(contentLength = offs, CACHE_BUFFER_SIZE);
-
-    if (!client.Read(offs + data.GetPointer(offs + len), len))
-      break;
-
-    len = client.GetLastReadCount();
-
-    offs += len;
-  }
-
-  return Load(data, options);
-}
-
-
-bool PXML::StartAutoReloadURL(const PURL & url, 
-                              const PTimeInterval & timeout, 
-                              const PTimeInterval & refreshTime,
-                              PXMLParser::Options options)
-{
-  if (url.IsEmpty()) {
-    autoLoadError = "Cannot auto-load empty URL";
-    return false;
-  }
-
-  PWaitAndSignal m(autoLoadMutex);
-  autoLoadTimer.Stop();
-
-  SetOptions(options);
-  autoloadURL      = url;
-  autoLoadWaitTime = timeout;
-  autoLoadError.MakeEmpty();
-  autoLoadTimer.SetNotifier(PCREATE_NOTIFIER(AutoReloadTimeout));
-
-  bool stat = AutoLoadURL();
-
-  autoLoadTimer = refreshTime;
-
-  return stat;
-}
-
-
-void PXML::AutoReloadTimeout(PTimer &, INT)
-{
-  PThread::Create(PCREATE_NOTIFIER(AutoReloadThread), "XmlReload");
-}
-
-
-void PXML::AutoReloadThread(PThread &, INT)
-{
-  PWaitAndSignal m(autoLoadMutex);
-  OnAutoLoad(AutoLoadURL());
-  autoLoadTimer.Reset();
-}
-
-
-void PXML::OnAutoLoad(bool PTRACE_PARAM(ok))
-{
-  PTRACE_IF(3, !ok, "XML\tFailed to load XML: " << GetErrorString());
-}
-
-
-bool PXML::AutoLoadURL()
-{
-  bool stat = LoadURL(autoloadURL, autoLoadWaitTime);
-  if (stat)
-    autoLoadError.MakeEmpty();
-  else 
-    autoLoadError = GetErrorString() + psprintf(" at line %i, column %i", GetErrorLine(), GetErrorColumn());
-  return stat;
-}
-
-
-bool PXML::StopAutoReloadURL()
-{
-  PWaitAndSignal m(autoLoadMutex);
-  autoLoadTimer.Stop();
-  return true;
-}
-
-#endif // P_HTTP
-
-
 bool PXML::Load(const PString & data, PXMLParser::Options options)
 {
   m_options = options;
@@ -513,6 +392,7 @@ bool PXML::Load(const PString & data, PXMLParser::Options options)
 
   {
     PXMLParser parser(options);
+    parser.SetMaxEntityLength(m_maxEntityLength);
     int done = 1;
     stat = parser.Parse(data, data.GetLength(), done) != 0;
   
@@ -699,6 +579,7 @@ void PXML::ReadFrom(istream & strm)
   rootMutex.Signal();
 
   PXMLParser parser(m_options);
+  parser.SetMaxEntityLength(m_maxEntityLength);
   while (strm.good()) {
     PString line;
     strm >> line;
@@ -998,6 +879,148 @@ bool PXML::LoadAndValidate(const PString & body, const PXML::ValidationInfo * va
 }
 
 
+///////////////////////////////////////////////////////
+
+#if P_HTTP
+
+PXML_HTTP::PXML_HTTP(int options, const char * noIndentElements)
+  : PXML(options, noIndentElements)
+{
+}
+
+
+bool PXML_HTTP::LoadURL(const PURL & url)
+{
+  return LoadURL(url, PMaxTimeInterval, PXMLParser::NoOptions);
+}
+
+
+bool PXML_HTTP::LoadURL(const PURL & url, const PTimeInterval & timeout, PXMLParser::Options options)
+{
+  if (url.IsEmpty()) {
+    m_errorString = "Cannot load empty URL";
+    m_errorLine = m_errorColumn = 0;
+    return false;
+  }
+
+  PTRACE(4, "XML\tLoading URL " << url);
+
+  PString data;
+  if (url.GetScheme() == "file") 
+    return LoadFile(url.AsFilePath());
+
+  PHTTPClient client;
+  PINDEX contentLength;
+  PMIMEInfo outMIME, replyMIME;
+
+  // make sure we do not hang around for ever
+  client.SetReadTimeout(timeout);
+
+  // get the resource header information
+  if (!client.GetDocument(url, outMIME, replyMIME)) {
+    m_errorString = "Cannot load URL ";
+    m_errorLine = m_errorColumn = 0;
+    m_errorString << '"' << url << '"';
+    return false;
+  }
+
+  // get the length of the data
+  if (replyMIME.Contains(PHTTPClient::ContentLengthTag()))
+    contentLength = (PINDEX)replyMIME[PHTTPClient::ContentLengthTag()].AsUnsigned();
+  else
+    contentLength = P_MAX_INDEX;
+
+  // download the resource into memory
+  PINDEX offs = 0;
+  for (;;) {
+    PINDEX len;
+    if (contentLength == P_MAX_INDEX)
+      len = CACHE_BUFFER_SIZE;
+    else if (offs == contentLength)
+      break;
+    else
+      len = PMIN(contentLength = offs, CACHE_BUFFER_SIZE);
+
+    if (!client.Read(offs + data.GetPointer(offs + len), len))
+      break;
+
+    len = client.GetLastReadCount();
+
+    offs += len;
+  }
+
+  return Load(data, options);
+}
+
+
+bool PXML_HTTP::StartAutoReloadURL(const PURL & url, 
+                                   const PTimeInterval & timeout, 
+                                   const PTimeInterval & refreshTime,
+                                   PXMLParser::Options options)
+{
+  if (url.IsEmpty()) {
+    autoLoadError = "Cannot auto-load empty URL";
+    return false;
+  }
+
+  PWaitAndSignal m(autoLoadMutex);
+  autoLoadTimer.Stop();
+
+  SetOptions(options);
+  autoloadURL      = url;
+  autoLoadWaitTime = timeout;
+  autoLoadError.MakeEmpty();
+  autoLoadTimer.SetNotifier(PCREATE_NOTIFIER(AutoReloadTimeout));
+
+  bool stat = AutoLoadURL();
+
+  autoLoadTimer = refreshTime;
+
+  return stat;
+}
+
+
+void PXML_HTTP::AutoReloadTimeout(PTimer &, INT)
+{
+  PThread::Create(PCREATE_NOTIFIER(AutoReloadThread), "XmlReload");
+}
+
+
+void PXML_HTTP::AutoReloadThread(PThread &, INT)
+{
+  PWaitAndSignal m(autoLoadMutex);
+  OnAutoLoad(AutoLoadURL());
+  autoLoadTimer.Reset();
+}
+
+
+void PXML_HTTP::OnAutoLoad(bool PTRACE_PARAM(ok))
+{
+  PTRACE_IF(3, !ok, "XML\tFailed to load XML: " << GetErrorString());
+}
+
+
+bool PXML_HTTP::AutoLoadURL()
+{
+  bool stat = LoadURL(autoloadURL, autoLoadWaitTime);
+  if (stat)
+    autoLoadError.MakeEmpty();
+  else 
+    autoLoadError = GetErrorString() + psprintf(" at line %i, column %i", GetErrorLine(), GetErrorColumn());
+  return stat;
+}
+
+
+bool PXML_HTTP::StopAutoReloadURL()
+{
+  PWaitAndSignal m(autoLoadMutex);
+  autoLoadTimer.Stop();
+  return true;
+}
+
+#endif // P_HTTP
+
+
 ///////////////////////////////////////////////////////
 //
 void PXMLObject::SetDirty()
