@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26644 $
+ * $Revision: 28604 $
  * $Author: rjongbloed $
- * $Date: 2011-11-06 19:51:26 -0600 (Sun, 06 Nov 2011) $
+ * $Date: 2012-11-27 01:41:14 -0600 (Tue, 27 Nov 2012) $
  */
 
 #ifdef __GNUC__
@@ -231,6 +231,18 @@ PString PURL::TranslateString(const PString & str, TranslationType type)
       safeChars += ":@&=+$,|";   // Section 3.3
       break;
 
+    case ParameterTranslation :
+      /* By strict RFC2396/3.3 this should be as for PathTranslation, but many
+         URI schemes have parameters of the form key=value so we don't allow
+         '=' character in the allowed set. Also, including one of "@,|" is
+         incompatible with some schemes, leave those out too. */
+      safeChars += ":&+$";
+      break;
+
+    case QuotedParameterTranslation :
+      safeChars += "[]/:@&=+$,|";
+      return str.FindSpan(safeChars) != P_MAX_INDEX ? str.ToLiteral() : str;
+
     default :
       break;    // Section 3.4, no reserved characters may be used
   }
@@ -893,14 +905,14 @@ void PURL::SetContents(const PString & str)
 }
 
 
-bool PURL::LoadResource(PString & str, const PString & requiredContentType)
+bool PURL::LoadResource(PString & str, const PString & requiredContentType) const
 {
   PURLLoader * loader = PURLLoaderFactory::CreateInstance(GetScheme());
   return loader != NULL && loader->Load(*this, str, requiredContentType);
 }
 
 
-bool PURL::LoadResource(PBYTEArray & data, const PString & requiredContentType)
+bool PURL::LoadResource(PBYTEArray & data, const PString & requiredContentType) const
 {
   PURLLoader * loader = PURLLoaderFactory::CreateInstance(GetScheme());
   return loader != NULL && loader->Load(*this, data, requiredContentType);
