@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26981 $
+ * $Revision: 28045 $
  * $Author: rjongbloed $
- * $Date: 2012-02-13 18:28:48 -0600 (Mon, 13 Feb 2012) $
+ * $Date: 2012-07-17 02:31:23 -0500 (Tue, 17 Jul 2012) $
  */
 
 #include <ptlib.h>
@@ -676,14 +676,18 @@ static const unsigned char PStringEscapeValue[] = { '\a', '\b', '\f', '\n', '\r'
 
 static void TranslateEscapes(const char * src, char * dst)
 {
-  if (*src == '"')
+  bool hadLeadingQuote = *src == '"';
+  if (hadLeadingQuote)
     src++;
 
   while (*src != '\0') {
     int c = *src++ & 0xff;
-    if (c == '"' && *src == '\0')
-      c  = '\0'; // Trailing '"' is ignored
-    else if (c == '\\') {
+    if (c == '"' && hadLeadingQuote) {
+      *dst = '\0'; // Trailing '"' and remaining string is ignored
+      break;
+    }
+
+    if (c == '\\') {
       c = *src++ & 0xff;
       for (PINDEX i = 0; i < PARRAYSIZE(PStringEscapeCode); i++) {
         if (c == PStringEscapeCode[i])
@@ -2439,6 +2443,7 @@ PINDEX PSortedStringList::InternalStringSelect(const char * str,
 ///////////////////////////////////////////////////////////////////////////////
 
 PStringSet::PStringSet(PINDEX count, char const * const * strarr, PBoolean caseless)
+  : BaseClass(true)
 {
   if (count == 0)
     return;
@@ -2456,11 +2461,28 @@ PStringSet::PStringSet(PINDEX count, char const * const * strarr, PBoolean casel
 
 
 PStringSet::PStringSet(const PString & str)
+  : BaseClass(true)
 {
   Include(str);
 }
 
 
+PStringSet::PStringSet(const PStringArray & strArray)
+  : BaseClass(true)
+{
+  for (PINDEX i = 0; i < strArray.GetSize(); ++i)
+    Include(strArray[i]);
+}
+
+
+PStringSet::PStringSet(const PStringList & strList)
+  : BaseClass(true)
+{
+  for (PStringList::const_iterator it = strList.begin(); it != strList.end(); ++it)
+    Include(*it);
+}
+
+
 void PStringSet::ReadFrom(istream & strm)
 {
   while (strm.good()) {
