@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
+ * $Revision: 26771 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 12:40:38 +0200 (mer 04 jun 2008) $
+ * $Date: 2011-12-07 00:34:45 -0600 (Wed, 07 Dec 2011) $
  */
 
 #ifdef __GNUC__
@@ -37,6 +37,7 @@
 
 #include <ptlib.h>
 #include <ptlib/pfactory.h>
+#include <ptlib/pprocess.h>
 #include <sstream>
 #include <ctype.h>
 #ifdef _WIN32
@@ -75,6 +76,12 @@ PFactoryBase::FactoryMap::~FactoryMap()
   }  
 }
 
+#if !P_USE_ASSERTS
+
+#define PAssertFunc(a, b, c, d) { }
+
+#else
+
 void PAssertFunc(const char * file,
                  int line,
                  const char * className,
@@ -140,14 +147,28 @@ void PAssertFunc(const char * file, int line, const char * className, const char
   PAssertFunc(str.str().c_str());
 }
 
-PObject::Comparison PObject::CompareObjectMemoryDirect(const PObject&obj) const
+#endif // !P_USE_ASSERTS
+
+PObject::Comparison PObject::CompareObjectMemoryDirect(const PObject & obj) const
 {
-  int retval = memcmp(this, &obj, sizeof(PObject));
+  return InternalCompareObjectMemoryDirect(this, &obj, sizeof(obj));
+}
+
+
+PObject::Comparison PObject::InternalCompareObjectMemoryDirect(const PObject * obj1,
+                                                               const PObject * obj2,
+                                                               PINDEX size)
+{
+  if (obj2 == NULL)
+    return PObject::LessThan;
+  if (obj1 == NULL)
+    return PObject::GreaterThan;
+  int retval = memcmp((const void *)obj1, (const void *)obj2, size);
   if (retval < 0)
-    return LessThan;
+    return PObject::LessThan;
   if (retval > 0)
-    return GreaterThan;
-  return EqualTo;
+    return PObject::GreaterThan;
+  return PObject::EqualTo;
 }
 
 
@@ -372,8 +393,7 @@ PMemoryHeap::~PMemoryHeap()
 
 #if defined(_WIN32)
   DeleteCriticalSection(&mutex);
-  extern void PWaitOnExitConsoleWindow();
-  PWaitOnExitConsoleWindow();
+  PProcess::Current().WaitOnExitConsoleWindow();
 #elif defined(P_PTHREADS)
   pthread_mutex_destroy(&mutex);
 #elif defined(P_VXWORKS)
@@ -811,12 +831,6 @@ PMemoryHeap::PMemoryHeap()
 PMemoryHeap::~PMemoryHeap()
 {
   _CrtMemDumpAllObjectsSince(&initialState);
-
-  if (hadCrtDumpLeak) {
-    extern void PWaitOnExitConsoleWindow();
-    PWaitOnExitConsoleWindow();
-  }
-
   _CrtSetDbgFlag(_CrtSetDbgFlag(_CRTDBG_REPORT_FLAG) & ~_CRTDBG_LEAK_CHECK_DF);
 }
 
@@ -857,6 +871,7 @@ void PMemoryHeap::Deallocate(void * ptr, const char * className)
 
 PMemoryHeap::Validation PMemoryHeap::Validate(const void * ptr, const char * className, ostream * /*strm*/)
 {
+  MemoryHeapInstance();
   if (!_CrtIsValidHeapPointer(ptr))
     return Bad;
 
@@ -870,12 +885,14 @@ PMemoryHeap::Validation PMemoryHeap::Validate(const void * ptr, const char * cla
 
 PBoolean PMemoryHeap::ValidateHeap(ostream * /*strm*/)
 {
+  MemoryHeapInstance();
   return _CrtCheckMemory();
 }
 
 
 PBoolean PMemoryHeap::SetIgnoreAllocations(PBoolean ignore)
 {
+  MemoryHeapInstance();
   int flags = _CrtSetDbgFlag(_CRTDBG_REPORT_FLAG);
   if (ignore)
     _CrtSetDbgFlag(flags & ~_CRTDBG_ALLOC_MEM_DF);
@@ -887,6 +904,7 @@ PBoolean PMemoryHeap::SetIgnoreAllocations(PBoolean ignore)
 
 void PMemoryHeap::DumpStatistics()
 {
+  MemoryHeapInstance();
   _CrtMemState state;
   _CrtMemCheckpoint(&state);
   _CrtMemDumpStatistics(&state);
@@ -901,24 +919,28 @@ void PMemoryHeap::DumpStatistics(ostream & /*strm*/)
 
 void PMemoryHeap::GetState(State & state)
 {
+  MemoryHeapInstance();
   _CrtMemCheckpoint(&state);
 }
 
 
 void PMemoryHeap::DumpObjectsSince(const State & state)
 {
+  MemoryHeapInstance();
   _CrtMemDumpAllObjectsSince(&state);
 }
 
 
 void PMemoryHeap::DumpObjectsSince(const State & state, ostream & /*strm*/)
 {
+  MemoryHeapInstance();
   DumpObjectsSince(state);
 }
 
 
 void PMemoryHeap::SetAllocationBreakpoint(DWORD objectNumber)
 {
+  MemoryHeapInstance();
   _CrtSetBreakAlloc(objectNumber);
 }
 
