@@ -7,9 +7,9 @@
  *
  * Contributor(s): Snark at GnomeMeeting
  *
- * $Revision: 26272 $
+ * $Revision: 26636 $
  * $Author: rjongbloed $
- * $Date: 2011-08-03 19:31:06 -0500 (Wed, 03 Aug 2011) $
+ * $Date: 2011-11-03 18:57:12 -0500 (Thu, 03 Nov 2011) $
  */
 
 #include <ptlib.h>
@@ -50,8 +50,6 @@
 #define PTPLUGIN_SUFFIX       "_ptplugin"
 #define PWPLUGIN_SUFFIX       "_pwplugin"
 
-PString PPluginManager::additionalPluginDirs = "";
-
 const char PDevicePluginServiceDescriptor::SeparatorChar = '\t';
 
 
@@ -61,9 +59,6 @@ class PluginLoaderStartup : public PProcessStartup
   public:
     void OnStartup();
     void OnShutdown();
-
-  protected:
-    std::vector<PPluginModuleManager *> managers;
 };
 
 
@@ -112,19 +107,27 @@ void PPluginManager::LoadPluginDirectory (const PDirectory & directory, const PS
 }
 
 
-void PPluginManager::AddPluginDirs(PString dirs)
+static PString & GetAdditionalPluginDirs()
 {
-  additionalPluginDirs += PATH_SEP + dirs;
+  static PString additionalPluginDirs;
+  return additionalPluginDirs;
+}
+
+
+bool PPluginManager::AddPluginDirs(const PString & dirs)
+{
+  GetAdditionalPluginDirs() += PATH_SEP + dirs;
+  return true;
 }
 
 
 PStringArray PPluginManager::GetPluginDirs()
 {
   PString env = ::getenv(ENV_PTLIB_PLUGIN_DIR);
-  if (env.IsEmpty()) 
+  if (env.IsEmpty())
     env = ::getenv(ENV_PWLIB_PLUGIN_DIR);
-  if (env.IsEmpty()) 
-    env = P_DEFAULT_PLUGIN_DIR + additionalPluginDirs;
+  if (env.IsEmpty())
+    env = P_DEFAULT_PLUGIN_DIR + GetAdditionalPluginDirs();
 
   // split into directories on correct seperator
   PStringArray dirs = env.Tokenise(PATH_SEP, true);
@@ -505,36 +508,27 @@ void PluginLoaderStartup::OnStartup()
   // load the actual DLLs, which will also load the system plugins
   PStringArray dirs = PPluginManager::GetPluginDirs();
   PPluginManager & mgr = PPluginManager::GetPluginManager();
-  PINDEX i;
-  for (i = 0; i < dirs.GetSize(); i++) 
+  for (PINDEX i = 0; i < dirs.GetSize(); i++) 
     mgr.LoadPluginDirectory(dirs[i]);
 
-  // load the plugin module managers, and construct the list of suffixes
+  // load the plugin module managers
   PFactory<PPluginModuleManager>::KeyList_T keyList = PFactory<PPluginModuleManager>::GetKeyList();
-  PFactory<PPluginModuleManager>::KeyList_T::const_iterator r;
-  for (r = keyList.begin(); r != keyList.end(); ++r) {
-    PPluginModuleManager * mgr = PFactory<PPluginModuleManager>::CreateInstance(*r);
-    if (mgr == NULL) {
-      PTRACE(1, "PLUGIN\tCannot create manager for plugins of type " << *r);
-    } else {
-      PTRACE(3, "PLUGIN\tCreated manager for plugins of type " << *r);
-      managers.push_back(mgr);
-    }
-  }
+  PFactory<PPluginModuleManager>::KeyList_T::const_iterator it;
+  for (it = keyList.begin(); it != keyList.end(); ++it)
+    PFactory<PPluginModuleManager>::CreateInstance(*it)->OnStartup();
 }
 
+
 void PluginLoaderStartup::OnShutdown()
 {
   PPluginManager::GetPluginManager().OnShutdown();
 
-  while (managers.begin() != managers.end()) {
-    std::vector<PPluginModuleManager *>::iterator r = managers.begin();
-    PPluginModuleManager * mgr = *r;
-    managers.erase(r);
-    mgr->OnShutdown();
-    delete mgr;
-  }
+  // load the plugin module managers, and construct the list of suffixes
+  PFactory<PPluginModuleManager>::KeyList_T keyList = PFactory<PPluginModuleManager>::GetKeyList();
+  PFactory<PPluginModuleManager>::KeyList_T::const_iterator it;
+  for (it = keyList.begin(); it != keyList.end(); ++it)
+    PFactory<PPluginModuleManager>::CreateInstance(*it)->OnShutdown();
 }
 
-PFACTORY_CREATE(PFactory<PProcessStartup>, PluginLoaderStartup, "PluginLoader", true);
+PFACTORY_CREATE(PProcessStartupFactory, PluginLoaderStartup, PLUGIN_LOADER_STARTUP_NAME, true);
 
