@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24994 $
+ * $Revision: 28050 $
  * $Author: rjongbloed $
- * $Date: 2011-01-04 17:13:51 -0600 (Tue, 04 Jan 2011) $
+ * $Date: 2012-07-18 02:15:30 -0500 (Wed, 18 Jul 2012) $
  */
 
 #include <ptlib.h>
@@ -223,7 +223,7 @@ void PSystemLogTarget::OutputToStream(ostream & stream, PSystemLog::Level level,
   PThread * thread = PThread::Current();
   PString threadName;
   if (thread == NULL)
-    threadName.sprintf("ThreadID" PTHREAD_ID_FMT, PThread::GetCurrentThreadId());
+    threadName.sprintf("Thread:" PTHREAD_ID_FMT, PThread::GetCurrentThreadId());
   else
     threadName = thread->GetThreadName();
   if (threadName.GetLength() <= 23)
