@@ -23,9 +23,9 @@
  *
  * Contributor(s): Mark Cooke (mpc@star.sr.bham.ac.uk)
  *
- * $Revision: 25752 $
+ * $Revision: 29328 $
  * $Author: rjongbloed $
- * $Date: 2011-05-11 21:50:32 -0500 (Wed, 11 May 2011) $
+ * $Date: 2013-03-26 18:23:37 -0500 (Tue, 26 Mar 2013) $
  */
 
 #ifdef __GNUC__
@@ -205,6 +205,26 @@ PVideoFrameInfo::PVideoFrameInfo(unsigned        width,
 }
 
 
+PObject::Comparison PVideoFrameInfo::Compare(const PObject & obj) const
+{
+  const PVideoFrameInfo & other = dynamic_cast<const PVideoFrameInfo &>(obj);
+
+  unsigned area = frameWidth*frameHeight;
+  unsigned otherArea = other.frameWidth*other.frameHeight;
+  if (area < otherArea)
+    return LessThan;
+  if (area > otherArea)
+    return GreaterThan;
+
+  if (frameRate < other.frameRate)
+    return LessThan;
+  if (frameRate > other.frameRate)
+    return GreaterThan;
+
+  return colourFormat.Compare(other.colourFormat);
+}
+
+
 void PVideoFrameInfo::PrintOn(ostream & strm) const
 {
   if (!colourFormat.IsEmpty())
