@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25079 $
+ * $Revision: 28468 $
  * $Author: rjongbloed $
- * $Date: 2011-01-20 12:23:32 -0600 (Thu, 20 Jan 2011) $
+ * $Date: 2012-10-04 23:20:30 -0500 (Thu, 04 Oct 2012) $
  */
 
 #include <ptlib.h>
@@ -795,33 +795,148 @@ PBoolean PSoundChannelWin32::OpenDevice(unsigned id)
   if (osError != MMSYSERR_NOERROR)
     return SetErrorValues(NotFound, osError|PWIN32ErrorFlag);
 
-  if (hMixer != NULL) {
-    line.cbStruct = sizeof(line);
-    if (mixerGetLineInfo((HMIXEROBJ)hMixer, &line, MIXER_OBJECTF_HMIXER | MIXER_GETLINEINFOF_COMPONENTTYPE) != MMSYSERR_NOERROR) {
-      mixerClose(hMixer);
-      hMixer = NULL;
+  opened = true;
+  os_handle = id;
+
+  if (hMixer == NULL) {
+    PTRACE(2, "WinSnd\tNo mixer available");
+    return true; // Still return true as have actual device
+  }
+
+  line.cbStruct = sizeof(line);
+  if ((osError = mixerGetLineInfo((HMIXEROBJ)hMixer, &line,
+            MIXER_OBJECTF_HMIXER | MIXER_GETLINEINFOF_COMPONENTTYPE)) != MMSYSERR_NOERROR) {
+    PTRACE(2, "WinSnd\tFailed to get mixer info, error=" << osError);
+  }
+  else {
+    bool haveControl = true;
+    MIXERLINECONTROLS controls;
+
+    if ((direction == Recorder)
+#ifndef _WIN32_WCE
+      && ((DWORD)(LOBYTE(LOWORD(GetVersion()))) < 6)
+#endif
+      ) { //5=XP/win2003
+      /* There is no "master" for the recording side, so need to find the
+         single selected input or at least individual microphone input
+         No need to do all of these on Vista/Win7/Win2008 as there is
+         a "master" for every input on those OS */
+
+      DWORD iConn = line.cConnections; //should save first
+      
+      //1)Attempt to find currently selected input
+      MIXERCONTROL muxControl;
+      muxControl.cbStruct = sizeof(muxControl);
+      controls.cbStruct = sizeof(controls);
+      controls.dwLineID = line.dwLineID;
+      controls.dwControlType = MIXERCONTROL_CONTROLTYPE_MUX;
+      controls.cControls = 1;
+      controls.pamxctrl = &muxControl;
+      controls.cbmxctrl = muxControl.cbStruct;
+      if ((osError = mixerGetLineControls((HMIXEROBJ)hMixer, &controls,
+                                          MIXER_OBJECTF_HMIXER | MIXER_GETLINECONTROLSF_ONEBYTYPE)) == MMSYSERR_NOERROR) {
+        MIXERCONTROLDETAILS mxcd;
+        mxcd.cbStruct = sizeof(mxcd);
+        mxcd.dwControlID = muxControl.dwControlID;
+        mxcd.cChannels = 1;
+        mxcd.cMultipleItems = muxControl.cMultipleItems;
+        mxcd.cbDetails = sizeof(MIXERCONTROLDETAILS_LISTTEXT);
+        std::vector<MIXERCONTROLDETAILS_LISTTEXT> mxcdlt(mxcd.cChannels * mxcd.cMultipleItems); 
+        mxcd.paDetails = &mxcdlt[0];
+        // Get the control list text items (input lines id's)
+        if ((osError = mixerGetControlDetails((HMIXEROBJ)hMixer, &mxcd,
+                                              MIXER_OBJECTF_HMIXER | MIXER_GETCONTROLDETAILSF_LISTTEXT)) == MMSYSERR_NOERROR) {
+          mxcd.cbDetails = sizeof(MIXERCONTROLDETAILS_BOOLEAN);
+          std::vector<MIXERCONTROLDETAILS_BOOLEAN> mxcdb(mxcd.cChannels * mxcd.cMultipleItems);
+          mxcd.paDetails = &mxcdb[0];
+          // Get the control values for line states (selected/not selected)
+          if ((osError = mixerGetControlDetails((HMIXEROBJ)hMixer, &mxcd,
+                                                MIXER_OBJECTF_HMIXER | MIXER_GETCONTROLDETAILSF_VALUE)) == MMSYSERR_NOERROR) {
+            DWORD iMult = mxcd.cMultipleItems;
+            while (iMult > 0) {
+              line.dwLineID = mxcdlt[--iMult].dwParam1;
+              if ((osError = mixerGetLineInfo((HMIXEROBJ)hMixer, &line,
+                                              MIXER_OBJECTF_HMIXER | MIXER_GETLINEINFOF_LINEID)) != MMSYSERR_NOERROR)
+                PTRACE(2, "WinSnd\tRecorder: Failed to get mixer info, error=" << osError);
+              else {
+                if (mxcdb[iMult].fValue == 1) {
+                  PTRACE(2, "WinSnd\tRecorder: Found selected input device: " << line.szName);
+                  break;
+                }
+
+                if (iMult == 0) {
+                  haveControl = false;
+                  PTRACE(2, "WinSnd\tRecorder: Failed to find selected input device");
+                }
+              }
+            }
+          }
+          else {
+            haveControl = false;
+            PTRACE(2, "WinSnd\tRecorder: failed to get info about states of input lines, error=" << osError);
+          }
+        }
+        else {
+          haveControl = false;
+          PTRACE(2, "WinSnd\tRecorder: failed to get info about input lines, error=" << osError);
+        }
+      }
+      else {
+        haveControl = false;
+        PTRACE(2, "WinSnd\tRecorder: failed to get mixer line mux control, error=" << osError);
+      }
+
+      //2) Attempt to find microphone input if not seccessfull with selected input
+      if (!haveControl) {
+        haveControl = true;
+        PTRACE(2, "WinSnd\tRecorder: trying to find microphone now...");
+        while (iConn > 0) {
+          line.dwSource = --iConn;
+          if ((osError = mixerGetLineInfo((HMIXEROBJ)hMixer, &line, MIXER_GETLINEINFOF_SOURCE)) != MMSYSERR_NOERROR)
+            PTRACE(2, "WinSnd\tFailed to get mixer info, error=" << osError);
+          else if (line.dwComponentType == MIXERLINE_COMPONENTTYPE_SRC_MICROPHONE) {
+            PTRACE(5, "WinSnd\tFfound microphone, source=" << iConn);
+            break;
+          }
+
+          if (iConn == 0) {
+            PTRACE(2, "WinSnd\tFailed to find microphone info");
+            haveControl = false;
+          }
+        }
+      }
     }
-    else {
+
+    if (haveControl) {
       volumeControl.cbStruct = sizeof(volumeControl);
 
-      MIXERLINECONTROLS controls;
       controls.cbStruct = sizeof(controls);
-      controls.dwLineID = line.dwLineID&0xffff;
+      controls.dwLineID = line.dwLineID;
       controls.dwControlType = MIXERCONTROL_CONTROLTYPE_VOLUME;
       controls.cControls = 1;
       controls.pamxctrl = &volumeControl;
       controls.cbmxctrl = volumeControl.cbStruct;
 
-      if (mixerGetLineControls((HMIXEROBJ)hMixer, &controls, MIXER_OBJECTF_HMIXER | MIXER_GETLINECONTROLSF_ONEBYTYPE) != MMSYSERR_NOERROR) {
-        mixerClose(hMixer);
-        hMixer = NULL;
+      if ((osError = mixerGetLineControls((HMIXEROBJ)hMixer, &controls,
+                MIXER_OBJECTF_HMIXER | MIXER_GETLINECONTROLSF_ONEBYTYPE)) == MMSYSERR_NOERROR) {
+        muteControl.cbStruct = sizeof(muteControl);
+        controls.dwControlType = MIXERCONTROL_CONTROLTYPE_MUTE;
+        controls.pamxctrl = &muteControl;
+        controls.cbmxctrl = muteControl.cbStruct;
+        if ((osError = mixerGetLineControls((HMIXEROBJ)hMixer, &controls,
+                MIXER_OBJECTF_HMIXER | MIXER_GETLINECONTROLSF_ONEBYTYPE)) != MMSYSERR_NOERROR) {
+          PTRACE(2, "WinSnd\tFailed to get mixer line mute control ("
+                 << (direction == Recorder ? "Recorder" : "Player") << "), error=" << osError);
+        }
+        return true;
       }
+      PTRACE(2, "WinSnd\tFailed to get mixer line control, error=" << osError);
     }
   }
 
-  opened = true;
-  os_handle = id;
-  return PTrue;
+  mixerClose(hMixer);
+  hMixer = NULL;
+  return true; // Still return true as have actual device
 }
 
 PBoolean PSoundChannelWin32::IsOpen() const
@@ -1392,8 +1507,7 @@ PBoolean PSoundChannelWin32::SetVolume(unsigned newVolume)
     volume.dwValue = volumeControl.Bounds.dwMaximum;
   else
     volume.dwValue = volumeControl.Bounds.dwMinimum +
-            (DWORD)((volumeControl.Bounds.dwMaximum - volumeControl.Bounds.dwMinimum) *
-                                                   log10(9.0*newVolume/MaxVolume + 1.0));
+            (DWORD)((volumeControl.Bounds.dwMaximum - volumeControl.Bounds.dwMinimum)*newVolume/MaxVolume);
   PTRACE(5, "WinSnd\tVolume set to " << newVolume << " -> " << volume.dwValue);
 
   MIXERCONTROLDETAILS details;
@@ -1432,9 +1546,59 @@ PBoolean PSoundChannelWin32::GetVolume(unsigned & oldVolume)
   if (result != MMSYSERR_NOERROR)
     return SetErrorValues(Miscellaneous, result|PWIN32ErrorFlag);
 
-  oldVolume = 100*(volume.dwValue - volumeControl.Bounds.dwMinimum)/(volumeControl.Bounds.dwMaximum - volumeControl.Bounds.dwMinimum);
+  oldVolume = MaxVolume*(volume.dwValue + 1 - volumeControl.Bounds.dwMinimum)/(volumeControl.Bounds.dwMaximum - volumeControl.Bounds.dwMinimum);
   return true;
 }
 
+
+bool PSoundChannelWin32::SetMute(bool newMute)
+{
+  if (!IsOpen() || hMixer == NULL)
+    return SetErrorValues(NotOpen, EBADF);
+
+  MIXERCONTROLDETAILS_UNSIGNED mute;
+  mute.dwValue = newMute;
+  PTRACE(5, "WinSnd\tMute set to " << newMute << " -> " << mute.dwValue);
+
+  MIXERCONTROLDETAILS details;
+  details.cbStruct = sizeof(details);
+  details.dwControlID = muteControl.dwControlID;
+  details.cChannels = 1;
+  details.cMultipleItems = 0;
+  details.cbDetails = sizeof(mute);
+  details.paDetails = &mute;
+
+  MMRESULT result = mixerSetControlDetails((HMIXEROBJ)hMixer, &details, MIXER_OBJECTF_HMIXER | MIXER_SETCONTROLDETAILSF_VALUE);
+  if (result != MMSYSERR_NOERROR)
+    return SetErrorValues(Miscellaneous, result|PWIN32ErrorFlag);
+
+  return true;
+}
+
+
+bool PSoundChannelWin32::GetMute(bool & oldMute)
+{
+  if (!IsOpen() || hMixer == NULL)
+    return SetErrorValues(NotOpen, EBADF);
+
+  MIXERCONTROLDETAILS_UNSIGNED mute;
+
+  MIXERCONTROLDETAILS details;
+  details.cbStruct = sizeof(details);
+  details.dwControlID = muteControl.dwControlID;
+  details.cChannels = 1;
+  details.cMultipleItems = 0;
+  details.cbDetails = sizeof(mute);
+  details.paDetails = &mute;
+
+  MMRESULT result = mixerGetControlDetails((HMIXEROBJ)hMixer, &details, MIXER_OBJECTF_HMIXER | MIXER_GETCONTROLDETAILSF_VALUE);
+  if (result != MMSYSERR_NOERROR)
+    return SetErrorValues(Miscellaneous, result|PWIN32ErrorFlag);
+
+  oldMute = mute.dwValue;
+  return true;
+}
+
+
 // End of File ///////////////////////////////////////////////////////////////
 
