@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26872 $
+ * $Revision: 29085 $
  * $Author: rjongbloed $
- * $Date: 2012-01-19 22:19:40 -0600 (Thu, 19 Jan 2012) $
+ * $Date: 2013-02-13 14:47:59 -0600 (Wed, 13 Feb 2013) $
  */
 
 #include <ptlib.h>
@@ -94,7 +94,6 @@ void PTime::SetFromFileTime(const FILETIME & timestamp)
   ULARGE_INTEGER i;
   i.HighPart = timestamp.dwHighDateTime;
   i.LowPart = timestamp.dwLowDateTime;
-  i.QuadPart;
 
   theTime = (time_t)(i.QuadPart/scale - delta);
   microseconds = (long)(i.QuadPart%scale/10);
@@ -250,14 +249,11 @@ unsigned PTimer::Resolution()
     return 1;
 
 #ifndef _WIN32_WCE
-  DWORD err = ::GetLastError();
   DWORD timeAdjustment;
   DWORD timeIncrement;
   BOOL timeAdjustmentDisabled;
   if (GetSystemTimeAdjustment(&timeAdjustment, &timeIncrement, &timeAdjustmentDisabled))
     return timeIncrement/10000;
-
-  err = ::GetLastError();
 #endif
 
   return 55;
@@ -763,7 +759,7 @@ UINT __stdcall PThread::MainFunction(void * threadPtr)
 */
 
   process.m_activeThreadMutex.Wait();
-  process.m_activeThreads[thread->threadId] = thread;
+  process.m_activeThreads[thread->m_threadId] = thread;
   process.m_activeThreadMutex.Signal();
 
   process.SignalTimerChange();
@@ -792,30 +788,28 @@ void PThread::Win32AttachThreadInput()
 {
 #ifndef _WIN32_WCE
   PProcess & process = PProcess::Current();
-  ::AttachThreadInput(threadId, ((PThread&)process).threadId, PTrue);
-  ::AttachThreadInput(((PThread&)process).threadId, threadId, PTrue);
+  ::AttachThreadInput(m_threadId, ((PThread&)process).m_threadId, PTrue);
+  ::AttachThreadInput(((PThread&)process).m_threadId, m_threadId, PTrue);
 #endif
 }
 
 
-PThread::PThread()
-  : autoDelete(false)
-  , m_isProcess(true)
-  , originalStackSize(0)
+PThread::PThread(bool isProcess)
+  : m_isProcess(isProcess)
+  , m_autoDelete(!isProcess)
+  , m_originalStackSize(0)
+  , m_threadId(GetCurrentThreadId())
   , threadHandle(GetCurrentThread())
-  , threadId(GetCurrentThreadId())
 {
-  if (!PProcess::IsInitialised())
+  if (isProcess)
     return;
 
-  m_isProcess = false;
-  autoDelete = true;
   DuplicateHandle(GetCurrentProcess(), threadHandle, GetCurrentProcess(), &threadHandle, 0, 0, DUPLICATE_SAME_ACCESS);
 
   PProcess & process = PProcess::Current();
 
   process.m_activeThreadMutex.Wait();
-  process.m_activeThreads[threadId] = this;
+  process.m_activeThreads[m_threadId] = this;
   process.m_activeThreadMutex.Signal();
 
   process.deleteThreadMutex.Wait();
@@ -828,28 +822,25 @@ PThread::PThread(PINDEX stackSize,
                  AutoDeleteFlag deletion,
                  Priority priorityLevel,
                  const PString & name)
-  : threadName(name)
-  , m_isProcess(false)
+  : m_isProcess(false)
+  , m_autoDelete(deletion == AutoDeleteThread)
+  , m_originalStackSize(stackSize)
+  , m_threadName(name)
 {
   PAssert(stackSize > 0, PInvalidParameter);
-  originalStackSize = stackSize;
-
-  autoDelete = deletion == AutoDeleteThread;
 
 #ifndef _WIN32_WCE
-  threadHandle = (HANDLE)_beginthreadex(NULL, stackSize, MainFunction,
-                                        this, CREATE_SUSPENDED, &threadId);
+  threadHandle = (HANDLE)_beginthreadex(NULL, stackSize, MainFunction, this, CREATE_SUSPENDED, &m_threadId);
 #else
    threadHandle = CreateThread(NULL, stackSize, 
-                               (LPTHREAD_START_ROUTINE)MainFunction,
-                               this, CREATE_SUSPENDED, (LPDWORD) &threadId);
+                       (LPTHREAD_START_ROUTINE)MainFunction, this, CREATE_SUSPENDED, (LPDWORD) &m_threadId);
 #endif
 
   PAssertOS(threadHandle != NULL);
 
   SetPriority(priorityLevel);
 
-  if (autoDelete) {
+  if (IsAutoDelete()) {
     PProcess & process = PProcess::Current();
     process.deleteThreadMutex.Wait();
     process.autoDeleteThreads.Append(this);
@@ -901,7 +892,7 @@ void PThread::CleanUp()
 
   PProcess & process = PProcess::Current();
   process.m_activeThreadMutex.Wait();
-  process.m_activeThreads.erase(threadId);
+  process.m_activeThreads.erase(m_threadId);
   process.m_activeThreadMutex.Signal();
 
   if (!IsTerminated())
@@ -914,19 +905,17 @@ void PThread::CleanUp()
 
 void PThread::Restart()
 {
-  if (!PAssert(originalStackSize != 0, "Cannot restart process/external thread") ||
+  if (!PAssert(m_originalStackSize != 0, "Cannot restart process/external thread") ||
       !PAssert(IsTerminated(), "Cannot restart running thread"))
     return;
 
   CleanUp();
 
 #ifndef _WIN32_WCE
-  threadHandle = (HANDLE)_beginthreadex(NULL,
-                         originalStackSize, MainFunction, this, 0, &threadId);
+  threadHandle = (HANDLE)_beginthreadex(NULL, m_originalStackSize, MainFunction, this, 0, &m_threadId);
 #else
-   threadHandle = CreateThread(NULL, originalStackSize, 
-                (LPTHREAD_START_ROUTINE) MainFunction,
-                  this, 0, (LPDWORD) &threadId);
+   threadHandle = CreateThread(NULL, m_originalStackSize, 
+                                  (LPTHREAD_START_ROUTINE)MainFunction, this, 0, (LPDWORD)&m_threadId);
 #endif
   PAssertOS(threadHandle != NULL);
 }
@@ -1016,17 +1005,17 @@ PBoolean PThread::IsSuspended() const
 
 void PThread::SetAutoDelete(AutoDeleteFlag deletion)
 {
-  PAssert(deletion != AutoDeleteThread || this != &PProcess::Current(), PLogicError);
+  PAssert(deletion != AutoDeleteThread || (!m_isProcess && this != &PProcess::Current()), PLogicError);
   bool newAutoDelete = (deletion == AutoDeleteThread);
-  if (autoDelete == newAutoDelete)
+  if (m_autoDelete == newAutoDelete)
     return;
 
-  autoDelete = newAutoDelete;
+  m_autoDelete = newAutoDelete;
 
   PProcess & process = PProcess::Current();
 
   process.deleteThreadMutex.Wait();
-  if (autoDelete)
+  if (m_autoDelete)
     process.autoDeleteThreads.Append(this);
   else {
     process.autoDeleteThreads.DisallowDeleteObjects();
@@ -1172,7 +1161,7 @@ void PProcess::HouseKeepingThread::Main()
 
 bool PProcess::SignalTimerChange()
 {
-  if (m_shuttingDown) 
+  if (!PAssert(IsInitialised(), PLogicError) || m_shuttingDown) 
     return false;
 
   deleteThreadMutex.Wait();
@@ -1430,9 +1419,9 @@ PBoolean PProcess::SetGroupName(const PString & groupname, PBoolean)
 }
 
 
-DWORD PProcess::GetProcessID() const
+PProcessIdentifier PProcess::GetCurrentProcessID()
 {
-  return GetCurrentProcessId();
+  return ::GetCurrentProcessId();
 }
 
 
