@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23931 $
+ * $Revision: 26406 $
  * $Author: rjongbloed $
- * $Date: 2010-01-13 06:19:08 -0600 (Wed, 13 Jan 2010) $
+ * $Date: 2011-09-12 02:15:55 -0500 (Mon, 12 Sep 2011) $
  */
 
 #pragma implementation "sockets.h"
@@ -58,11 +58,14 @@
 #define ifr_netmask ifr_addr
 
 #include <net/if_dl.h>
+
+#if !defined(P_IPHONEOS)
 #include <net/if_types.h>
 #include <net/route.h>
+#endif
 
 #include <netinet/in.h>
-#if !defined(P_QNX)
+#if !defined(P_QNX)  && !defined(P_IPHONEOS)
 #include <netinet/if_ether.h>
 #endif
 
@@ -206,23 +209,32 @@ PBoolean PSocket::os_connect(struct sockaddr * addr, PINDEX size)
 
 PBoolean PSocket::os_accept(PSocket & listener, struct sockaddr * addr, PINDEX * size)
 {
-  if (!listener.PXSetIOBlock(PXAcceptBlock, listener.GetReadTimeout()))
-    return SetErrorValues(listener.GetErrorCode(), listener.GetErrorNumber());
+  int new_fd;
+  while ((new_fd = ::accept(listener.GetHandle(), addr, (socklen_t *)size)) < 0) {
+    switch (errno) {
+      case EINTR :
+        break;
 
 #if defined(E_PROTO)
-  for (;;) {
-    int new_fd = ::accept(listener.GetHandle(), addr, (socklen_t *)size);
-    if (new_fd >= 0)
-      return ConvertOSError(os_handle = SetNonBlocking(new_fd));
-
-    if (errno != EPROTO)
-      return ConvertOSError(-1);
-
-    PTRACE(3, "PTLib\tAccept on " << sock << " failed with EPROTO - retrying");
-  }
-#else
-  return ConvertOSError(os_handle = SetNonBlocking(::accept(listener.GetHandle(), addr, (socklen_t *)size)));
+      case EPROTO :
+        PTRACE(3, "PTLib\tAccept on " << listener << " failed with EPROTO - retrying");
+        break;
 #endif
+
+      case EWOULDBLOCK :
+        if (listener.GetReadTimeout() > 0) {
+          if (listener.PXSetIOBlock(PXAcceptBlock, listener.GetReadTimeout()))
+            break;
+          return SetErrorValues(listener.GetErrorCode(), listener.GetErrorNumber());
+        }
+        // Next case
+
+      default :
+        return ConvertOSError(-1, LastReadError);
+    }
+  }
+
+  return ConvertOSError(os_handle = SetNonBlocking(new_fd));
 }
 
 
@@ -562,10 +574,10 @@ PBoolean PTCPSocket::Read(void * buf, PINDEX maxLen)
 #if P_HAS_RECVMSG
 
 PBoolean PSocket::os_recvfrom(
-      void * buf,     // Data to be written as URGENT TCP data.
-      PINDEX len,     // Number of bytes pointed to by <CODE>buf</CODE>.
+      void * buf,
+      PINDEX len,
       int    flags,
-      sockaddr * addr, // Address from which the datagram was received.
+      sockaddr * addr,
       PINDEX * addrlen)
 {
   lastReadCount = 0;
@@ -592,7 +604,7 @@ PBoolean PSocket::os_recvfrom(
   // read a packet 
   int r = ::recvmsg(os_handle, &readData, flags);
   if (r == -1) {
-    PTRACE(5, "PTLIB\trecvmsg returned error " << r);
+    PTRACE(5, "PTLIB\trecvmsg returned error " << errno);
     ::recvmsg(os_handle, &readData, MSG_ERRQUEUE);
   }
 
@@ -726,15 +738,11 @@ PBoolean PEthSocket::Connect(const PString & interfaceName)
   fakeMacHeader = PFalse;
   ipppInterface = PFalse;
 
-  if (strncmp("eth", interfaceName, 3) == 0)
-    medium = Medium802_3;
-  else if (strncmp("lo", interfaceName, 2) == 0)
+  if (strncmp("lo", interfaceName, 2) == 0)
     medium = MediumLoop;
-  else if (strncmp("sl", interfaceName, 2) == 0) {
-    medium = MediumWan;
-    fakeMacHeader = PTrue;
-  }
-  else if (strncmp("ppp", interfaceName, 3) == 0) {
+  else if (strncmp("sl", interfaceName, 2) == 0 ||
+           strncmp("wlan", interfaceName, 4) == 0 ||
+           strncmp("ppp", interfaceName, 3) == 0) {
     medium = MediumWan;
     fakeMacHeader = PTrue;
   }
@@ -742,12 +750,8 @@ PBoolean PEthSocket::Connect(const PString & interfaceName)
     medium = MediumWan;
     ipppInterface = PTrue;
   }
-#ifdef P_RTEMS
-  else if (strncmp(RTEMS_BSP_NETWORK_DRIVER_NAME, interfaceName, 3) == 0)
-    medium = Medium802_3;
-#endif
   else
-    return SetErrorValues(NotFound, ENOENT);
+    medium = Medium802_3;
 
 #if defined(SIO_Get_MAC_Address) 
   PUDPSocket ifsock;
@@ -1032,43 +1036,71 @@ PString PIPSocket::GetGatewayInterface(int version)
 
 PBoolean PIPSocket::GetRouteTable(RouteTable & table)
 {
-  PTextFile procfile;
-  if (!procfile.Open("/proc/net/route", PFile::ReadOnly))
-    return false;
+  table.RemoveAll();
 
   PString strLine;
-  bool ok = procfile.ReadLine(strLine);
-  if (ok)
-    ok = procfile.ReadLine(strLine);
+  PTextFile procfile;
 
-  int nTime = 0;
-  while (ok) {
-    // Ignore heading line or remainder of route line
-    strLine.RightTrim();
-    PStringArray strList = strLine.Tokenise("\t");
-    if (strList.GetSize() < 8)
-      return nTime > 0 ? true : false;
-
-    PString iface = strList[0];
-    unsigned net_addr = strList[1].AsUnsigned(16);
-    unsigned dest_addr = strList[2].AsUnsigned(16);
-    int metric= strList[6].AsInteger(10);
-    unsigned net_mask = strList[7].AsUnsigned(16);
-
-    nTime ++;
-    RouteEntry * entry = new RouteEntry(net_addr);
-    entry->net_mask = net_mask;
-    entry->destination = dest_addr;
-    entry->interfaceName = iface;
-    entry->metric = metric;
-    table.Append(entry);
-    ok = procfile.ReadLine(strLine);
+  if (procfile.Open("/proc/net/route", PFile::ReadOnly) && procfile.ReadLine(strLine)) {
+    // Ignore heading line above
+    while (procfile.ReadLine(strLine)) {
+      char iface[20];
+      uint32_t net_addr, dest_addr, net_mask;
+      int flags, refcnt, use, metric;
+      if (sscanf(strLine, "%s%x%x%x%u%u%u%x",
+                 iface, &net_addr, &dest_addr, &flags, &refcnt, &use, &metric, &net_mask) == 8) {
+        RouteEntry * entry = new RouteEntry(net_addr);
+        entry->net_mask = net_mask;
+        entry->destination = dest_addr;
+        entry->interfaceName = iface;
+        entry->metric = metric;
+        table.Append(entry);
+      }
+    }
   }
 
-  return nTime > 0;
+#if P_HAS_IPV6
+  if (procfile.Open("/proc/net/ipv6_route", PFile::ReadOnly)) {
+    while (procfile.ReadLine(strLine)) {
+      PStringArray tokens = strLine.Tokenise(" \t", false);
+      if (tokens.GetSize() == 10) {
+        // 0 = dest_addr
+        // 1 = net_mask (cidr in hex)
+        // 2 = src_addr
+        // 4 = next_hop
+        // 5 = metric
+        // 6 = refcnt
+        // 7 = use
+        // 8 = flags
+        // 9 = device name
+
+        BYTE net_addr[16];
+        for (size_t i = 0; i < sizeof(net_addr); ++i)
+          net_addr[i] = tokens[0].Mid(i*2, 2).AsUnsigned(16);
+
+        BYTE dest_addr[16];
+        for (size_t i = 0; i < sizeof(dest_addr); ++i)
+          dest_addr[i] = tokens[4].Mid(i*2, 2).AsUnsigned(16);
+
+        RouteEntry * entry = new RouteEntry(Address(sizeof(net_addr), net_addr));
+        entry->destination = Address(sizeof(dest_addr), dest_addr);
+        entry->interfaceName = tokens[9];
+        entry->metric = tokens[5].AsUnsigned(16);
+		BYTE net_mask[16];
+		bzero(net_mask, sizeof(net_mask));
+		for(size_t i = 0; i < tokens[1].AsUnsigned(16) / 4; ++i)
+			net_mask[i/2] = (i % 2 == 0) ? 0xf0 : 0xff;
+        entry->net_mask = Address(sizeof(net_mask), net_mask);
+        table.Append(entry);
+      }
+    }
+  }
+#endif
+
+  return !table.IsEmpty();
 }
 
-#elif defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_NETBSD) || defined(P_MACOSX) || defined(P_QNX) 
+#elif (defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_NETBSD) || defined(P_MACOSX) || defined(P_QNX)) && !defined(P_IPHONEOS)
 
 PBoolean process_rtentry(struct rt_msghdr *rtm, char *ptr, unsigned long *p_net_addr,
                      unsigned long *p_net_mask, unsigned long *p_dest_addr, int *p_metric);
@@ -1165,10 +1197,10 @@ PBoolean process_rtentry(struct rt_msghdr *rtm, char *ptr, unsigned long *p_net_
   if ((~rtm->rtm_flags&RTF_LLINFO)
 #if defined(P_NETBSD) || defined(P_QNX)
         && (~rtm->rtm_flags&RTF_CLONED)     // Net BSD has flag one way
-#elif !defined(P_OPENBSD)
-        && (~rtm->rtm_flags&RTF_WASCLONED)  // Free BSD/MAC has it another
+#elif !defined(P_OPENBSD) && !defined(P_FREEBSD)
+        && (~rtm->rtm_flags&RTF_WASCLONED)  // MAC has it another
 #else
-                                            // Open BSD does not have it at all!
+                                            // Open/Free BSD does not have it at all!
 #endif
      ) {
 
@@ -1652,7 +1684,165 @@ PIPSocket::RouteTableDetector * PIPSocket::CreateRouteTableDetector()
   return new NetLinkRouteTableDetector();
 }
 
-#else
+#elif defined(P_IPHONEOS)
+
+#include <netdb.h>
+#include <sys/time.h>
+#include <net/if.h>
+#include <netinet/in.h>
+#include <arpa/inet.h>
+
+#include <SystemConfiguration/SystemConfiguration.h>
+#include <SystemConfiguration/SCNetworkReachability.h>
+
+#define kSCNetworkReachabilityOptionNodeName	CFSTR("nodename")
+
+/*!
+	@constant kSCNetworkReachabilityOptionServName
+	@discussion A CFString that will be passed to getaddrinfo(3).  An acceptable
+		value is either a decimal port number or a service name listed in
+		services(5).
+ */
+#define kSCNetworkReachabilityOptionServName	CFSTR("servname")
+
+/*!
+	@constant kSCNetworkReachabilityOptionHints
+	@discussion A CFData wrapping a "struct addrinfo" that will be passed to
+		getaddrinfo(3).  The caller can supply any of the ai_family,
+		ai_socktype, ai_protocol, and ai_flags structure elements.  All
+		other elements must be 0 or the null pointer.
+ */
+#define kSCNetworkReachabilityOptionHints	CFSTR("hints")
+
+class ReachabilityRouteTableDetector : public PIPSocket::RouteTableDetector
+{
+	SCNetworkReachabilityRef	target_async;
+
+   public:
+	SCNetworkReachabilityRef _setupReachability(SCNetworkReachabilityContext *context)
+	{
+		struct sockaddr_in		sin;
+		struct sockaddr_in6		sin6;
+		SCNetworkReachabilityRef	target	= NULL;
+
+		bzero(&sin, sizeof(sin));
+		sin.sin_len    = sizeof(sin);
+		sin.sin_family = AF_INET;
+
+		bzero(&sin6, sizeof(sin6));
+		sin6.sin6_len    = sizeof(sin6);
+		sin6.sin6_family = AF_INET6;
+
+		const char *anchor = "apple.com";
+
+		if (inet_aton(anchor, &sin.sin_addr) == 1) {
+
+			target = SCNetworkReachabilityCreateWithAddress(NULL, (struct sockaddr *)&sin);
+			
+		} else if (inet_pton(AF_INET6, anchor, &sin6.sin6_addr) == 1) {
+			char	*p;
+
+			p = strchr(anchor, '%');
+			if (p != NULL) {
+				sin6.sin6_scope_id = if_nametoindex(p + 1);
+			}
+
+			target = SCNetworkReachabilityCreateWithAddress(NULL, (struct sockaddr *)&sin6);
+			
+		} else {
+			
+		target = SCNetworkReachabilityCreateWithName(NULL, anchor);
+				
+#if	!TARGET_OS_IPHONE
+		if (CFDictionaryGetCount(options) > 0) {
+
+			target = SCNetworkReachabilityCreateWithOptions(NULL, options);
+
+		} else {
+
+			SCPrint(TRUE, stderr, CFSTR("Must specify nodename or servname\n"));
+			return NULL;
+		}
+		CFRelease(options);
+#endif			
+		}
+
+		return target;
+	}
+	  
+	static void callout(SCNetworkReachabilityRef target, SCNetworkReachabilityFlags flags, void *info)
+	{
+		struct tm	tm_now;
+		struct timeval	tv_now;
+
+		(void)gettimeofday(&tv_now, NULL);
+		(void)localtime_r(&tv_now.tv_sec, &tm_now);
+
+		PTRACE(1, psprintf("Reachability changed at: %2d:%02d:%02d.%03d, now it is %sreachable",
+			tm_now.tm_hour,
+			tm_now.tm_min,
+			tm_now.tm_sec,
+			tv_now.tv_usec / 1000,
+			flags & kSCNetworkReachabilityFlagsReachable? "" : "not ")
+			);
+
+		ReachabilityRouteTableDetector* d = (ReachabilityRouteTableDetector*) info;	
+		d->Cancel();
+	}
+
+	ReachabilityRouteTableDetector()
+		: RouteTableDetector(),
+		target_async(NULL)
+	{
+		SCNetworkReachabilityContext	context	= { 0, NULL, NULL, NULL, NULL };
+		
+		target_async = _setupReachability(&context);
+		if (target_async == NULL) {
+			PTRACE(1, psprintf("  Could not determine status: %s\n", SCErrorString(SCError())));
+			return;
+		}
+		
+		context.info = (void*) this;
+		
+		if (!SCNetworkReachabilitySetCallback(target_async, ReachabilityRouteTableDetector::callout, &context)) {
+			PTRACE(1, psprintf("SCNetworkReachabilitySetCallback() failed: %s\n", SCErrorString(SCError())));
+			return;
+		}
+
+		if (!SCNetworkReachabilityScheduleWithRunLoop(target_async, CFRunLoopGetCurrent(), kCFRunLoopDefaultMode)) {
+			PTRACE(1, psprintf("SCNetworkReachabilityScheduleWithRunLoop() failed: %s\n", SCErrorString(SCError()) ) );
+			return;
+		}
+	}
+   
+	~ReachabilityRouteTableDetector()
+	{
+		if(target_async != NULL)
+			CFRelease(target_async);
+	}
+
+    bool Wait(const PTimeInterval & timeout)
+    {
+		m_cancel.Wait(timeout);
+		return PTrue;
+    }
+
+    void Cancel()
+    {
+      m_cancel.Signal();
+    }
+
+  private:
+    PSyncPoint m_cancel;
+	PBoolean m_continue;
+};
+
+PIPSocket::RouteTableDetector * PIPSocket::CreateRouteTableDetector()
+{
+	return new ReachabilityRouteTableDetector();
+}
+
+#else // P_HAS_NETLINK, elif defined(P_IPHONEOS)
 
 class DummyRouteTableDetector : public PIPSocket::RouteTableDetector
 {
@@ -1677,46 +1867,11 @@ PIPSocket::RouteTableDetector * PIPSocket::CreateRouteTableDetector()
   return new DummyRouteTableDetector();
 }
 
-#endif
+#endif // P_HAS_NETLINK, elif defined(P_IPHONEOS)
 
 
-// fe800000000000000202e3fffe1ee330 02 40 20 80     eth0
-// 00000000000000000000000000000001 01 80 10 80       lo
-
 PBoolean PIPSocket::GetInterfaceTable(InterfaceTable & list, PBoolean includeDown)
 {
-#if P_HAS_IPV6
-  // build a table of IPV6 interface addresses
-  typedef std::map<PString, PString> IP6ListType;
-  IP6ListType ip6Ifaces;
-  {
-    FILE * file;
-    int dummy;
-    int addr[16];
-    char ifaceName[255];
-    if ((file = fopen("/proc/net/if_inet6", "r")) != NULL) {
-      while (fscanf(file, "%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x %x %x %x %x %255s\n",
-              &addr[0],  &addr[1],  &addr[2],  &addr[3], 
-              &addr[4],  &addr[5],  &addr[6],  &addr[7], 
-              &addr[8],  &addr[9],  &addr[10], &addr[11], 
-              &addr[12], &addr[13], &addr[14], &addr[15], 
-             &dummy, &dummy, &dummy, &dummy, ifaceName) != EOF) {
-        PString addrStr(
-          psprintf("%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x",
-              addr[0],  addr[1],  addr[2],  addr[3], 
-              addr[4],  addr[5],  addr[6],  addr[7], 
-              addr[8],  addr[9],  addr[10], addr[11], 
-              addr[12], addr[13], addr[14], addr[15]
-          )
-        );
-        PString iface(ifaceName);
-        ip6Ifaces.insert(IP6ListType::value_type(ifaceName, addrStr));
-      }
-      fclose(file);
-    }
-  }
-#endif
-
   PUDPSocket sock;
 
   PBYTEArray buffer;
@@ -1747,10 +1902,10 @@ PBoolean PIPSocket::GetInterfaceTable(InterfaceTable & list, PBoolean includeDow
 #endif
       struct ifreq ifReq;
 #if !defined(P_NETBSD)
-      memcpy(&ifReq, ifName, sizeof(ifreq));
+          memcpy(&ifReq, ifName, sizeof(ifreq));
 #else
-      memset(&ifReq, 0, sizeof(ifReq));
-      strncpy(ifReq.ifr_name, ifa->ifa_name, sizeof(ifReq.ifr_name) - 1);
+          memset(&ifReq, 0, sizeof(ifReq));
+          strncpy(ifReq.ifr_name, ifa->ifa_name, sizeof(ifReq.ifr_name) - 1);
 #endif
 
       if (ioctl(sock.GetHandle(), SIOCGIFFLAGS, &ifReq) >= 0) {
@@ -1761,10 +1916,8 @@ PBoolean PIPSocket::GetInterfaceTable(InterfaceTable & list, PBoolean includeDow
           PString macAddr;
 #if defined(SIO_Get_MAC_Address)
           memcpy(&ifReq, ifName, sizeof(ifreq));
-          if (ioctl(sock.GetHandle(), SIO_Get_MAC_Address, &ifReq) >= 0) {
-            PEthSocket::Address a((BYTE *)ifReq.ifr_macaddr);
-            macAddr = (PString)a;
-          }
+          if (ioctl(sock.GetHandle(), SIO_Get_MAC_Address, &ifReq) >= 0)
+            macAddr = PEthSocket::Address((BYTE *)ifReq.ifr_macaddr);
 #endif
 
 #if !defined(P_NETBSD)
@@ -1806,18 +1959,8 @@ PBoolean PIPSocket::GetInterfaceTable(InterfaceTable & list, PBoolean includeDow
 #endif
                   break;
               }
-#if P_HAS_IPV6
-              PString ip6Addr;
-              IP6ListType::const_iterator r = ip6Ifaces.find(name);
-              if (r != ip6Ifaces.end())
-                ip6Addr = r->second;
-#endif
               if (i >= list.GetSize())
-                list.Append(PNEW InterfaceEntry(name, addr, mask, macAddr
-#if P_HAS_IPV6
-                , ip6Addr
-#endif
-                ));
+                list.Append(PNEW InterfaceEntry(name, addr, mask, macAddr));
             }
           }
         }
@@ -1834,6 +1977,41 @@ PBoolean PIPSocket::GetInterfaceTable(InterfaceTable & list, PBoolean includeDow
 #if !defined(P_NETBSD)
   }
 #endif
+
+#if P_HAS_IPV6
+  // build a table of IPV6 interface addresses
+  // fe800000000000000202e3fffe1ee330 02 40 20 80     eth0
+  // 00000000000000000000000000000001 01 80 10 80       lo
+  FILE * file;
+  int dummy;
+  int addr[16];
+  char ifaceName[255];
+  if ((file = fopen("/proc/net/if_inet6", "r")) != NULL) {
+    while (fscanf(file, "%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x %x %x %x %x %255s\n",
+            &addr[0],  &addr[1],  &addr[2],  &addr[3], 
+            &addr[4],  &addr[5],  &addr[6],  &addr[7], 
+            &addr[8],  &addr[9],  &addr[10], &addr[11], 
+            &addr[12], &addr[13], &addr[14], &addr[15], 
+           &dummy, &dummy, &dummy, &dummy, ifaceName) != EOF) {
+      BYTE bytes[16];
+      for (PINDEX i = 0; i < 16; i++)
+        bytes[i] = addr[i];
+
+      PString macAddr;
+#if defined(SIO_Get_MAC_Address)
+      struct ifreq ifReq;
+      memset(&ifReq, 0, sizeof(ifReq));
+      strncpy(ifReq.ifr_name, ifaceName, sizeof(ifReq.ifr_name) - 1);
+      if (ioctl(sock.GetHandle(), SIO_Get_MAC_Address, &ifReq) >= 0)
+        macAddr = PEthSocket::Address((BYTE *)ifReq.ifr_macaddr);
+#endif
+
+      list.Append(PNEW InterfaceEntry(ifaceName, Address(16, bytes), Address::GetAny(6), macAddr));
+    }
+    fclose(file);
+  }
+#endif
+
   return PTrue;
 }
 
