@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21219 $
+ * $Revision: 27623 $
  * $Author: rjongbloed $
- * $Date: 2008-10-02 08:52:00 +0000 (Thu, 02 Oct 2008) $
+ * $Date: 2012-05-14 19:03:37 -0500 (Mon, 14 May 2012) $
  */
 
 #include <sys/resource.h>
@@ -42,7 +42,7 @@ extern int debug_mpthreads;
 PDECLARE_CLASS(PHouseKeepingThread, PThread)
   public:
     PHouseKeepingThread()
-      : PThread(1000, NoAutoDeleteThread, NormalPriority, "Housekeeper")
+      : PThread(1000, NoAutoDeleteThread, HighestPriority, "Housekeeper")
       { closing = PFalse; Resume(); }
 
     void Main();
@@ -441,6 +441,9 @@ long PThread::PX_ThreadStart(void * arg)
 
 void PProcess::SignalTimerChange()
 {
+  if (!PAssert(IsInitialised(), PLogicError) || m_shuttingDown) 
+    return false;
+
   if (housekeepingThread == NULL) {
 #if PMEMORY_CHECK
     PBoolean oldIgnoreAllocations = PMemoryHeap::SetIgnoreAllocations(PTrue);
